import warnings

import astropy.units as u
import celerite
import corner
import emcee
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stat
from astropy.timeseries import LombScargle
from astropy.visualization import quantity_support
from celerite import terms
from matplotlib import gridspec
from scipy.optimize import curve_fit, differential_evolution, minimize

mpl.rcParams['xtick.minor.visible'] = True
mpl.rcParams['xtick.top'] = True
mpl.rcParams['xtick.direction'] = 'in'

mpl.rcParams['ytick.minor.visible'] = True
mpl.rcParams['ytick.right'] = True
mpl.rcParams['ytick.direction'] = 'in'

mpl.rcParams["figure.autolayout"] = False

mpl.rcParams['savefig.dpi'] = 300
mpl.rcParams['savefig.format'] = 'pdf'


quantity_support()

##############################################################
###################### ASSIST FUNCTIONS ######################
##############################################################

def celerite_fit(x, y, yerr, kernel, nwalkers, nburn, nsamp,
                 solver='minimize', suppress_warnings=True, jitter=True):

    """Fit time-series data to a given Gaussian process kernel using celerite.


    Parameters
    ----------

    x : list of float
        Time values of the data.

    y : list of float
        Values of the data.

    yerr : list of float
        Uncertainties in the data.

    kernel : celerite.terms.Term
        Kernel to fit to the data.

    nwalkers : int
        Number of walkers to use in the MCMC.

    nburn : int
        Number of burn-in steps to use in the MCMC.

    nsamp : int
        Number of samples to use in the MCMC.

    solver : str, optional
        Solver to use for the GP fit. Options are "minimize" for ``scipy.optimize.minimize`` and
        "diff_evo" for ``scipy.optimize.differential_evolution``. Default is "minimize".

    suppress_warnings : bool, optional
        Suppress warnings from the GP fit. Default is ``True``.

    jitter : bool, optional
        If true, fit for a noise (jitter) term in the GP. Default is ``True``.



    Returns
    -------

    samples : array_like
        Samples of the kernel parameters from the MCMC fit.

    gp : celerite.GP
        GP object used for the fit.

    statuses : list of bool
        Status of the fit to the light curve. There are three statuses given:

        - baseline_good
            If ``True``, tau > baseline/10.

        - cadence_good
            If ``True``, tau > mean cadence of the light curve.

        - stn_good
            If ``True``, the DRW sigma parameter is greater than the noise in the light curve.

    """


    if suppress_warnings:
        warnings.filterwarnings("ignore")

    #Set up gp for celerite
    gp = celerite.GP(kernel, np.mean(y.value), fit_mean=False)
    gp.compute(x.value, yerr.value)


    # Define a cost function
    def neg_ll(params, y, gp):
        gp.set_parameter_vector(params)
        return -gp.log_likelihood(y)

    def grad_neg_ll(params, y, gp):
        gp.set_parameter_vector(params)
        return -gp.grad_log_likelihood(y)[1]

    # Define the log probablity
    def log_probability(params):
        gp.set_parameter_vector(params)
        lp = gp.log_prior()
        if not np.isfinite(lp):
            return -np.inf
        return gp.log_likelihood(y) + lp


    #Bounds and values for input parameters
    init_params = gp.get_parameter_vector()
    bounds = gp.get_parameter_bounds()

    #Get soln from extrema of maximum likelihood
    if solver == 'minimize':
        soln = minimize(neg_ll, init_params, jac=grad_neg_ll,
                        method="L-BFGS-B", bounds=bounds, args=(y.value, gp))
        initial = np.array(soln.x)
    elif solver == 'diff_evo':
        soln = differential_evolution(neg_ll, bounds=bounds, args=(y.value, gp))
        initial = np.array(soln.x)


    #Set parameter vector to fit params
    gp.set_parameter_vector(initial)

    #Set up MCMC
    ndim = len(initial)
    sampler = emcee.EnsembleSampler(nwalkers, ndim, log_probability)

    #Run burn-in on the MCMC
    p0 = initial + 1e-8 * np.random.randn(nwalkers, ndim)
    p0, lp, _ = sampler.run_mcmc(p0, nburn)

    #Run the MCMC
    sampler.reset()
    sampler.run_mcmc(p0, nsamp)


    #Get samples from the posterior dist
    samples = sampler.flatchain

    #Get median values for the samples from the posterior
    s = np.median(samples, axis=0)
    gp.set_parameter_vector(s)

    tau = 1/np.exp(s[1])
    sig_drw = np.sqrt( np.exp(s[0])/2 )

    if jitter is True:
        sig_n = np.exp(s[2])
    else:
        sig_n = 0

    baseline = x[-1] - x[0]
    if tau < .1*baseline.value:
        baseline_good = True
    else:
        baseline_good = False


    cadence = np.mean(np.diff(x.value))
    if tau > cadence:
        cadence_good = True
    else:
        cadence_good = False

    sig_drw = np.exp( s[1] )
    dy = np.mean(yerr.value)

    if sig_drw**2 > sig_n**2 + dy**2:
        stn_good = True
    else:
        stn_good = False

    return samples, gp, [baseline_good, cadence_good, stn_good]



def MCMC_fit(x, y, yerr, nwalkers=32, nburn=300, nsamp=1000,
             solver='minimize', suppress_warnings=True, jitter=True,
             clip=True):

    """Fit time-series data to a DRW using celerite and emcee.


    Parameters
    ----------

    x : list of astropy.units.Quantity
        Time values of the data.

    y : list of astropy.units.Quantity
        Values of the data.

    yerr : list of astropy.units.Quantity
        Uncertainties on the data.

    nwalkers : int, optional
        Number of walkers to use in the MCMC. Default is 32.

    nburn : int, optional
        Number of burn-in steps to use in the MCMC. Default is 300.

    nsamp : int, optional
        Number of samples to use in the MCMC. Default is 1000.

    solver : str, optional
        Solver to use for the GP fit. Options are "minimize" for ``scipy.optimize.minimize`` and
        "diff_evo" for ``scipy.optimize.differential_evolution``. Default is "minimize".

    suppress_warnings : bool, optional
        Suppress warnings from the GP fit. Default is ``True``.

    jitter : bool, optional
        If true, fit for a noise (jitter) term in the GP. Default is ``True``.

    clip : bool, optional
        If ``True``, clip data points too close together in time (< 1e-8 days). Default is ``True``.



    Returns
    -------

    samples : list of float
        Samples of the kernel parameters from the MCMC fit

    gp : celerite.GP
        GP object used for the fit

    statuses : list of bool
        Status of the fit to the light curve. There are three statuses given:

        - baseline_good
            If True, tau > baseline/10

        - cadence_good
            If true, tau > mean cadence of the light curve

        - stn_good
            If true, the DRW sigma parameter is greater than the noise in the light curve

    """


    #Set up GP in celerite
    baseline = x[-1] - x[0]


    #Bounds and value for a
    min_prec = np.min(yerr.value)
    amp = np.max( y.value + yerr.value ) - np.min(y.value - yerr.value)

    amin = np.log(.001*min_prec)
    amax = np.log(10*amp)

    aval = np.mean([amin, amax])


    #Bounds and value for c
    if clip is True:
        min_cadence = np.clip(np.min( np.diff(x.value) ), 1e-8, None)
    else:
        min_cadence = np.min( np.diff(x.value) )

    cmin = np.log( 1/10/baseline.value )
    cmax = np.log(1/min_cadence)

    cval = np.mean([cmin, cmax])

    bounds = dict(log_a=(amin, amax), log_c=(cmin, cmax))
    kernel = terms.RealTerm(log_a=aval, log_c=cval, bounds=bounds )


    if jitter is True:
        #Bounds and value for s (jitter)
        smin = -10
        smax = np.log(amp)
        sval = np.mean([smin, smax])

        bounds = dict(log_sigma=(smin, smax))
        kernel += terms.JitterTerm(log_sigma=sval, bounds=bounds)

    samples, gp, statuses = celerite_fit(x, y, yerr, kernel, nwalkers,
                                         nburn, nsamp, solver,
                                         suppress_warnings, jitter)

    return samples, gp, statuses



def psd_from_gp(fLS, powerLS, gp, samples, baseline):

    """Calculate the PSD of a light curve using the DRW fit from celerite.
    This assumes that a Lomb-Scargle periodogram has been made from the
    light curve, giving a list of frequencies and powers.


    Parameters
    ----------

    fLS : list of astropy.units.Quantity
        Frequencies from the Lomb-Scargle periodogram.

    powerLS : list of astropy.units.Quantity
        Powers from the Lomb-Scargle periodogram.

    gp : celerite.GP
        GP object used for the fit.

    samples : array_like
        Samples of the kernel parameters from the MCMC fit.

    baseline : astropy.units.Quantity
        Baseline of the light curve.



    Returns
    -------

    f_eval : list of astropy.units.Quantity
        Frequencies at which the PSD is evaluated

    psd_credint : list of astropy.units.Quantity
        An (n,3) array of the PSD, with the first columns being
        the 16th percentile, the second column being the median,
        and the third column being the 84th percentile.

    """


    #Get freqs
    f_eval = np.logspace( np.log10(fLS[0].value), np.log10(fLS[-1].value), 200 )

    psd_samples = np.empty( (len(f_eval), len(samples)) )
    kernel = gp.kernel

    #Get the PSD for each parameter-set in the sample
    for i, s in enumerate(samples):
        gp.set_parameter_vector(s)
        psd_samples[:, i] = kernel.get_psd(2*np.pi*f_eval)/(2*np.pi)


    #Get credibility intervals (16, 50, 84)
    psd_credint = np.empty((len(f_eval), 3))
    psd_credint[:, 0] = np.percentile(psd_samples, 16, axis=1)
    psd_credint[:, 2] = np.percentile(psd_samples, 84, axis=1)
    psd_credint[:, 1] = np.median(psd_samples, axis=1)


    #Normalize the PSD
    f_norm = np.max(powerLS.value[fLS.value>1/(2*np.pi*0.2*baseline.value)])/psd_credint[0, 1]
    #Max[ power[ f > 1/2pi/baseline/5 ] ]

    psd_credint[:, 0] = psd_credint[:, 0]*f_norm
    psd_credint[:, 2] = psd_credint[:, 2]*f_norm
    psd_credint[:, 1] = psd_credint[:, 1]*f_norm

    return f_eval, psd_credint





def binLS(fLS, powerLS_samp, num_bins):

    """Bin the Lomb-Scargle periodogram by frequency.

    Parameters
    ----------
    fLS : list of astropy.units.Quantity
        Frequencies from the Lomb-Scargle periodogram.

    powerLS_samp : list of astropy.units.Quantity
        Samples of the Lomb-Scargle periodogram.

    num_bins : int
        Number of bins to use for the binned periodogram.



    Returns
    -------

    binCenters : list of astropy.units.Quantity
        Bin centers for the binned periodogram.

    bin_vals : list of astropy.units.Quantity
        The values of the binned periodogram.

    bin_errs : list of astropy.units.Quantity
        The upper error on the binned periodogram.

    lower_err : list of astropy.units.Quantity
        The lower error on the binned periodogram.

    """


    #Get the credibility intervals for each point on the PSD
    bins_credint = np.empty((len(fLS), 3))
    bins_credint[:, 0] = np.percentile(powerLS_samp, 16, axis=0)
    bins_credint[:, 2] = np.percentile(powerLS_samp, 84, axis=0)
    bins_credint[:, 1] = np.median(powerLS_samp, axis=0)

    binEdges = np.logspace( np.log10(fLS[0].value), np.log10(fLS[-1].value), num_bins+1)

    #Get data for bins (16th, 50th, 84th percentile for each bin mean)
    draws_binned = np.empty((len(binEdges)-1, 3))
    draws_binned[:,0], _, _ = stat.binned_statistic(fLS, bins_credint[:, 0],
                                                                     statistic='mean',
                                                                     bins=binEdges)
    draws_binned[:,2], _, _ = stat.binned_statistic(fLS, bins_credint[:, 2],
                                                                     statistic='mean',
                                                                     bins=binEdges)
    draws_binned[:,1], _, _ = stat.binned_statistic(fLS, bins_credint[:, 1],
                                                                     statistic='mean',
                                                                     bins=binEdges)

    #Get error (upper and lower) for each bin value
    bin_err = np.array([draws_binned[:, 1] - draws_binned[:, 0],
                        draws_binned[:, 2] - draws_binned[:, 1]])

    binEdges[0] *= .99


    binCenters = []

    for i in range(len(binEdges) - 1):

        #Get center of the bin
        cent = (np.log10(binEdges[i]) + np.log10(binEdges[i+1]) )/2

        binCenters.append( 10**cent )

    bin_vals = draws_binned[:, 1]
    lower_err = bin_err[1]
    bin_err = bin_err[0]

    binCenters = np.concatenate( ([fLS[0].value], binCenters ) )
    bin_vals = np.concatenate( ([bin_vals[0]], bin_vals ) )
    bin_err = np.concatenate( ([0.], bin_err ) )
    lower_err = np.concatenate( ([0.], lower_err ) )

    binCenters = np.append( binCenters, fLS[-1].value )
    bin_vals = np.append( bin_vals, bin_vals[-1] )
    bin_err = np.append(bin_err, 0.)
    lower_err = np.append(lower_err, 0.)


    #Mask out infinite bins
    mask_finite = np.isfinite(bin_vals)
    binCenters = np.array(binCenters)[mask_finite]
    bin_vals = np.array(bin_vals)[mask_finite]
    bin_err = np.array(bin_err)[mask_finite]
    lower_err = np.array(lower_err)[mask_finite]

    return binCenters, bin_vals, bin_err, lower_err




def smoothly_broken_power_law(f, A=1, f_br=1e-3, alpha=0, beta=2):

    """A smoothly broken power law:

    .. math:: P(f) = \frac{A}{(f / f_{br})^\alpha + (f/f_{br})^\beta}



    Parameters
    ----------

    f : list of float
        Frequencies at which to evaluate the power law.

    A : float
        The amplitude of the power law.

    f_br : float
        The break frequency of the power law.

    alpha : float
        The slope of the power law below the break frequency.

    beta : float
        The slope of the power law above the break frequency.



    Returns
    -------

    power : list of float
        The power law.

    """

    return A/((f/f_br)**alpha + (f/f_br)**beta)



def psd_sbpl(f, psd, err, p0, bounds):

    """Generate a smoothly broken power law fit to the PSD.


    Parameters
    ----------

    f : list of float
        Frequencies for the input PSD.

    psd : list of float
        The input PSD.

    err : list of float
        The error on the input PSD.

    p0 : list of float
        The initial guess for the parameters of the smoothly broken power law.

    bounds : list of float
        The bounds on the parameters of the smoothly broken power law.




    Returns
    -------

    fit_vals : list of float
        The best fit parameters for the smoothly broken power law.

    fit_errs : list of float
        The errors on the best fit parameters for the smoothly broken power law.

    """


    try:
        soln = curve_fit(smoothly_broken_power_law, f, psd,
                         sigma=err, p0=p0, bounds=bounds, maxfev=10000)
        fit_vals = soln[0]
        fit_err = np.sqrt(np.diag(soln[1]))
    except:
        soln = curve_fit(smoothly_broken_power_law, f, psd,
                         sigma=None, p0=None, bounds=bounds, maxfev=10000)
        fit_vals = soln[0]
        fit_err = np.sqrt(np.diag(soln[1]))

    return fit_vals, fit_err


def psd_data(x, y, yerr, samples, gp, nsamp=20):


    """Generate all PSD data for the summary plot.


    Parameters
    ----------

    x : list of astropy.units.Quantity
        The light curve times.

    y : list of astropy.units.Quantity
        The light curve values.

    yerr : list of astropy.units.Quantity
        The uncertainty in the light curve.

    samples : list of float
        The samples from the MCMC fit.

    gp : celerite.GP
        The GP object used to fit the light curve.

    nsamp : int
        The number of samples of the Lomb-Scargle periodogram to use.



    Returns
    -------

    fLS : list of astropy.unit.Quantity
        The frequencies for the Lomb-Scargle periodogram.

    powerLS : list of astropy.unit.Quantity
        The power for the Lomb-Scargle periodogram.

    f_eval : list of astropy.unit.Quantity
        The frequencies for the PSD from the celerite fit.

    psd_credint : list of astropy.unit.Quantity
        The (16th, 50th, 84th) percentiles of the PSD from the celerite fit.

    bin_vals : list of astropy.unit.Quantity
        The values of the Lomb-Scargle periodogram.

    bin_err : list of astropy.unit.Quantity
        The upper error on the binned Lomb-Scargle periodogram.

    lower_err : list of astropy.unit.Quantity
        The lower error on the binned Lomb-Scargle periodogram.

    binCenters : list of astropy.unit.Quantity
        The centers of the bins for the binned Lomb-Scargle periodogram.

    fit_vals : list of astropy.unit.Quantity
        The best fit parameters for the smoothly broken power law fit to
        the Lomb-Scargle periodogram.

    fit_errs : list of astropy.unit.Quantity
        The errors on the best fit parameters for the smoothly broken
        power law fit to the Lomb-Scargle periodogram.

    """

    baseline = x[-1] - x[0]
    powerLS_samps = []


    for _ in range(nsamp):
        y_samp = y.value + np.random.normal(0, yerr.value)
        y_samp *= y.unit

        #Get Lomb-Scargle
        fLS, powerLS = LombScargle(x, y_samp, yerr).autopower(normalization='psd')

        #Put power into lightkurve units (flux variance / frequency unit)
        fs = (1./(np.min(np.diff(x.value)[np.diff(x)>0])))
        powerLS *= 2/(len(x)*fs)


        ind = np.argsort(fLS.value)
        fLS = fLS[ind]
        powerLS = powerLS[ind]

        powerLS_samps.append(powerLS)

    f_eval, psd_credint = psd_from_gp(fLS, powerLS, gp, samples, baseline)

    binCenters, bin_vals, bin_err, lower_err = binLS(fLS, powerLS_samps, 15)


    s = np.median(samples, axis=0)
    tau = 1/np.exp(s[1])
    f = 1/2/np.pi/tau

    bounds = np.array([(.1*np.nanmax(bin_vals), 10*np.nanmax(bin_vals)),
                       (fLS[0].value, fLS[-1].value),
                       (-3., 0.), (.1, 5.)]).T
    p0 = [np.nanmax(bin_vals), f, 0., 2.]

    fit_vals, fit_err = psd_sbpl(binCenters[1:-4], bin_vals[1:-4],
                                 None, p0, bounds)

    return fLS, powerLS, f_eval, psd_credint, \
           bin_vals, bin_err, binCenters, lower_err, \
           fit_vals, fit_err


##############################################################
##############################################################
##############################################################

def drw_flag(times, data, error,
             target=None, fname=None,
             nwalkers=32, nburn=300, nsamp=1000,
             nsig=1, jitter=True, clip=True,
             plot=True):


    """Fits the light curve to a DRW model using celerite with MCMC sampling from emcee.
    celerite will then predict the light curve with the best fit parameters. Points lying above
    or below the mean prediction by ``nsig`` standard deviations will be flagged as outliers in
    an output mask.


    Parameters
    ----------

    times : list of astropy.units.Quantity
        Times for the light curve.

    data : list of astropy.units.Quantity
        Data for the light curve.

    error : list of astropy.units.Quantity
        Uncertainty in the light curve.

    nwalkers : int, optional
        Number of walkers for the MCMC sampler. Default is 32.

    nburn : int, optional
        Number of burn-in steps for the MCMC sampler. Default is 300.

    nsamp : int, optional
        Number of samples for the MCMC sampler. Default is 1000.

    nsig : int, optional
        Number of standard deviations above or below the mean prediction to flag as an outlier.
        Default is 1.

    jitter : bool, optional
        If True, will fit for a noise (jitter) term in the data. Default is ``True``.

    clip : bool, optional
        If true, will clip data points which are too close in time to another data point. Specifically,
        will remove data points that are closer than 1e-8 days apart. Default is ``True``.

    target : str, optional
        The name of the target. Default is ``None``.

    fname : str, optional
        The name of the file to output the resulting plot. If ``None``, the plot will
        not be saved. Default is ``None``.

    plot : bool, optional
        If true, the plot will be shown. Default is ``True``.



    Returns
    --------

    res : dict
        The output of the DRW fitting and rejection. The keys are:

        * mask : array_like
            The DRW rejection mask. Values marked as True are outliers.

        * tau : array_like
            The DRW tau chain from the MCMC sampler.

        * sigma : array_like
            The DRW sigma chain from the MCMC sampler.

        * fit_x : array_like
            The times for the predicted light curve.

        * fit_y : array_like
            The predicted light curve.

        * fit_yerr : array_like
            The uncertainty in the predicted light curve.

        * jitter : array_like
            The jitter chain from the MCMC sampler. Only present if ``jitter`` is True.


    .. note:: The input times, data, and error must be astropy.units.Quantity objects.

    """

    assert times.shape == data.shape == error.shape
    assert len(times.shape) == 1


    if type(data[0]) == u.Quantity:
        data_unit = data[0].unit

    flag_mask = np.zeros( times.shape, dtype=bool )


    #Assume 'times' and 'fluxes' have the same shape

    #Fit to the DRW model
    samples, gp, statuses = MCMC_fit(times, data, error,
                                         nwalkers=nwalkers, nburn=nburn, nsamp=nsamp,
                                         jitter=jitter, clip=clip)


    fig, ax = plot_outcome(times, data, error, samples, gp, data_unit,
                                nsig=nsig, target=target, show_mean=True,
                                filename=fname, jitter=jitter, show=plot)

    plt.cla()
    plt.clf()
    plt.close()

    tau_vals = 1/np.exp(samples[:, 1])
    sig_vals = np.sqrt( np.exp(samples[:, 0])/2 )

    if jitter:
        jitter_vals = np.exp(samples[:, 2])


    #Get fit to light curve data
    baseline = times[-1] - times[0]
    extra_t = int(baseline.value//10)

    t = np.linspace( times[0].value - extra_t, times[-1].value + extra_t, 1000 ).tolist()
    for i in range(len(times)):
        t.append(times[i].value)

    sort_ind = np.argsort(t)
    t = np.array(t)[sort_ind]

    mu, var = gp.predict(data.value, t, return_var=True)
    std = np.sqrt(var)

    mu_flag = []
    for i in range(len(times)):
        ind = np.argwhere(t == times[i].value).T[0][0]
        mu_flag.append(mu[ind])

    #Reject if data is beyond nsig*sig of fit mean
    flag_mask = np.abs(data.value - mu_flag) > nsig*error.value

    res = {
        'mask': flag_mask,
        'tau' : tau_vals,
        'sigma': sig_vals,
        'fit_x': t,
        'fit_y': mu,
        'fit_err': std
    }

    if jitter:
        res['jitter'] = jitter_vals

    return res

##############################################################
######################## PLOT FUNCTIONS ######################
##############################################################


def plot_outcome(x, y, yerr, samples, gp, unit, nsig=0,
                 target=None, show_mean=True,
                 filename=None, jitter=True, show=False):

    """Generate summary image for the DRW fit. Contains a plot of the input light curve along with the DRW fit, a corner plot of the DRW parameters, and a plot of the PSD (data-based and fit-based).


    Parameters
    ----------

    x : list of astropy.units.Quantity
        The times of the light curve.

    y : list of astropy.units.Quantity
        The light curve.

    yerr : list of astro.units.Quantity
        The uncertainty in the light curve.

    samples : array_like
        The output MCMC samples from the DRW fit.

    gp : celerite.GP
        The GP object used to fit the DRW

    unit : str, astropy.units.Unit
        The unit of the light curve.

    nsig : int, optional
        The number of standard deviations away from the mean DRW fit at which to
        consider a point an outlier. Default is 0 (i.e. no points are outliers).

    target : str, optional
        The name of the target. Default is ``None``.

    show_mean : bool, optional
        If True, will plot the mean of the DRW fit. Default is True.

    filename : str, optional
        The name of the file to save the image to. Default is ``None``.

    jitter : bool, optional
        Whether or not the jitter term was included in the DRW fit. Default is True.

    show : bool, optional
        Whether or not to show the image. Default is False.




    Returns
    -------

    fig : matplotlib.figure.Figure
        The figure containing the summary image

    axs : list of matplotlib.axes.Axes
        The axes of the figure

    """

    #Plot probability dists of output params

    #tau = 1/c
    #sig = sqrt(a/2)

    baseline = x[-1] - x[0]
    extra_t = int(baseline.value//10)

    if unit == u.mag:
        unit_label = 'Magnitude'
    elif unit == u.dimensionless_unscaled:
        unit_label = 'Flux'
    else:
        unit_label = 'Flux [' + str(unit) + ']'

    if target is None:
        target_label = ''
    else:
        target_label = 'Light Curve of ' + target

    tau_vals = 1/np.exp(samples[:, 1])
    sig_vals = np.sqrt( np.exp(samples[:, 0])/2 )

    if jitter is True:
        n=3
        jitter_vals = np.exp(samples[:, 2])
        sample_vals = np.vstack((np.log10(sig_vals), np.log10(tau_vals), np.log10(jitter_vals) )).T
        labels = [r'$\log_{10}\ (\sigma_{\rm DRW})$',
                  r'$\log_{10}\ (\tau_{\rm DRW})$',
                  r'$\log_{10}\ (\sigma_n)$']
    else:
        n=2
        sample_vals = np.vstack((np.log10(sig_vals), np.log10(tau_vals) )).T
        labels = [r'$\log_{10}\ (\sigma_{\rm DRW})$',
                  r'$\log_{10}\ (\tau_{\rm DRW})$']


    #This is an array of the following: [[tau, sig, jit], [tau, sig, jit], ..., [tau, sig, jit]]

    fig, axs = plt.subplots(n,n, figsize=(5,5))

    titles = []

    sig = np.log10(np.median(sig_vals))
    sig_err_l = sig - np.log10( np.percentile(sig_vals,16) )
    sig_err_u = np.log10( np.percentile(sig_vals,84) ) - sig
    sig_title = r'$' + '{:.2f}'.format(sig) + '^{' + ' +{:.2f}'.format(sig_err_u) +  '}_{' + '-{:.2f}'.format(sig_err_l) + '}$'
    titles.append(sig_title)

    tau = np.log10(np.median(tau_vals))
    tau_err_l = tau - np.log10( np.percentile(tau_vals,16) )
    tau_err_u = np.log10( np.percentile(tau_vals,84) ) - tau
    tau_title = r'$' + '{:.2f}'.format(tau) + '^{' + ' +{:.2f}'.format(tau_err_u) +  '}_{' + '-{:.2f}'.format(tau_err_l) + '}$'
    titles.append(tau_title)

    if jitter is True:
        jit = np.log10(np.median(jitter_vals))
        jit_err_l = jit - np.log10( np.percentile(jitter_vals,16) )
        jit_err_u = np.log10( np.percentile(jitter_vals,84) ) - jit
        jit_title = r'$' + '{:.2f}'.format(jit) + '^{' + ' +{:.2f}'.format(jit_err_u) +  '}_{' + '-{:.2f}'.format(jit_err_l) + '}$'
        titles.append(jit_title)


    #Make corner plot
    fig = corner.corner(sample_vals, labels=labels,
                        quantiles=[.16, .84], use_math_text=True, titles=titles, fig=fig,
                        label_kwargs=dict(fontsize=12), title_kwargs=dict(fontsize=17))

    axs = np.array(fig.axes).reshape((n, n))

    for i in range(n):
        for j in range(n):
            ax = axs[i,j]
            ax.tick_params('both',labelsize=12)
            ax.tick_params(axis='both', which='both', direction='in')
            ax.tick_params(axis='both', which='major', length=6)
            ax.tick_params(axis='both', which='minor', length=3)


    axs[0,0].title.set_text(sig_title)
    axs[0,0].title.set_fontsize(17)

    axs[1,1].title.set_text(tau_title)
    axs[1,1].title.set_fontsize(17)
    if jitter is True:
        axs[2,2].title.set_text(jit_title)
        axs[2,2].title.set_fontsize(17)

    #Red out bad tau regions
    axs[1,1].axvspan(np.log10(0.2*baseline.value), np.log10(100*baseline.value), color= "red", zorder=-5, alpha=0.2)

    if jitter is True:
        axs[2,1].axvspan(np.log10(0.2*baseline.value), np.log10(100*baseline.value), color= "red", zorder=-5, alpha=0.2)

    axs[1,0].axhspan(np.log10(0.2*baseline.value), np.log10(100*baseline.value), color= "red", zorder=-5, alpha=0.2)

    gs = gridspec.GridSpec(ncols=4, nrows=4, figure=fig)


    #--------------------------------------------------------------------------
    #Plot data + posterior probability

    ax1 = fig.add_subplot(gs[0, :])

    box = ax1.get_position()
    box.x0 = box.x0 + 0.2
    box.x1 = box.x1 + 1.0
    box.y0 = box.y0 + 0.4
    box.y1 = box.y1 + 0.9
    ax1.set_position(box)


    t = np.linspace(x[0].value - extra_t, x[-1].value+extra_t, 1000)
    mu, var = gp.predict(y.value, t, return_var=True)
    std = np.sqrt(var)

    mu_flag, var_flag = gp.predict(y.value, x.value, return_var=True)

    flag_mask = np.abs(y.value - mu_flag) > nsig*yerr.value
    ax1.errorbar( (x.value - x[0].value)[flag_mask],
                 y.value[flag_mask], yerr.value[flag_mask],
                 fmt='.', color='DodgerBlue', capsize=1., alpha=.4, ms=7)
    ax1.errorbar( (x.value - x[0].value)[~flag_mask],
                y.value[~flag_mask], yerr.value[~flag_mask],
                fmt='.k', capsize=1., alpha=.7, ms=8)


    if show_mean is True:
        ax1.plot(t-x[0].value, mu, color='orange', zorder=-1)
    ax1.fill_between(t-x[0].value, mu+std, mu-std, color='orange', alpha=.3)

    ax1.set_xlabel('Time [' + str(x.unit) + ']', fontsize=20)
    ax1.set_ylabel(unit_label, fontsize=20)
    ax1.set_title(target_label, fontsize=21)

    ax1.tick_params(which='major', length=6)
    ax1.tick_params(which='minor', length=3)

    ax1.set_xlim(-extra_t/2, baseline.value + extra_t/2)

    if unit_label == 'Magnitude':
        ax1.invert_yaxis()

    ax1.tick_params('both', labelsize=17)

    #--------------------------------------------------------------------------
    #Plot LS PSD

    fLS, powerLS, f_eval, psd_credint, bin_vals, bin_err, binCenters, lower_err, fit_vals, fit_err = psd_data(x, y, yerr, samples, gp)

    ax2 = fig.add_subplot(gs[:, -1])
    fig.set_size_inches([6,6])

    # Move the subplot over
    box = ax2.get_position()
    box.x0 = box.x0 + 0.4
    box.x1 = box.x1 + 1.2
    ax2.set_position(box)

    #Plot
    ax2.loglog(fLS.value, powerLS.value, alpha=.3, label='Data-based PSD')

    #--------------------------------------------------------------------------
    #Get PSD from the celerite posterior dist

    #Fill the PSD b/w the 16th and 84th percentiles
    ax2.fill_between(f_eval, psd_credint[:, 2], psd_credint[:, 0],
                     alpha=0.3, label='Model PSD', color='orange')

    #--------------------------------------------------------------------------
    #Bin Lomb Scargle

    ax2.errorbar(binCenters, bin_vals, yerr=(lower_err, bin_err),
                 marker=None, drawstyle='steps-mid', color='k',
                 linewidth=1.0, capsize=3, label='Binned PSD')
    ax2.legend(loc='upper right', fontsize=15)

    #--------------------------------------------------------------------------
    #Fit PSD to a smoothly broken power law

    sbpl_dat = smoothly_broken_power_law(fLS.value, fit_vals[0], fit_vals[1], fit_vals[2], fit_vals[3])
    ax2.plot(fLS.value, sbpl_dat, color='red', label='SBPL Fit')

    #--------------------------------------------------------------------------
    #Plot values for f_br

    f_br = fit_vals[1]
    f_br_err = fit_err[1]

    f_br_plus = f_br + f_br_err
    f_br_minus = f_br - f_br_err

    if f_br_minus < 0.0 and np.isfinite(f_br_plus) is True:
        ls_diff = np.log10(f_br_plus) - np.log10(f_br)
        f_br_minus = 10**( np.log10(f_br) - ls_diff )


    f_br_val = psd_credint[:, 2][0]

    if np.isfinite(f_br_plus) is True:
        #Dat for f_br line
        t = np.logspace(np.log10(f_br_minus), np.log10(f_br_plus), 100)
        dat = np.full(len(t), f_br_val)
        ax2.plot(t, dat, color='red', linewidth=2)

        arrow_head = (f_br, 10**(np.log10(f_br_val)+.2) )
        arrow_base = (f_br, 10**(np.log10(f_br_val)+.6) )
        ax2.errorbar([f_br], [arrow_base[1]], yerr=[arrow_base[1] - arrow_head[1]],
                     uplims=True, color='red', elinewidth=2, capthick=2)

    #--------------------------------------------------------------------------
    #Red out bad freqs

    ax2.axvspan( np.min(fLS.value) , 1/(2*np.pi*0.2*baseline.value), color='red', alpha=.3)

    fmax = (1./(2*np.pi*np.mean(np.diff(x)[np.diff(x)>0])))
    ax2.axvspan(fmax.value, np.max(fLS.value), color='red', alpha=0.2)


    ax2.set_xlabel(r'Frequency [' + str(x.unit) + '$^{-1}$]', fontsize=18)
    ax2.set_ylabel(r'Power', fontsize=18)

    ax2.set_ylim( np.min(powerLS.value) )
    ax2.set_xlim(np.min(fLS.value), np.max(fLS.value))

    ax2.set_xscale('log', nonpositive='clip')
    ax2.set_yscale('log', nonpositive='clip')

    ax2.tick_params('both', labelsize=15)
    ax2.tick_params('both', which='both', direction='in')
    ax2.tick_params('both', which='major', length=8)
    ax2.tick_params('both', which='minor', length=3)

    fig = plt.gcf()

    if filename is not None:
        fig.savefig(f'{filename}', bbox_inches='tight')

    if show is True:
        plt.show()

    return fig, axs



import matplotlib as mpl
import matplotlib.gridspec as gridspec
import matplotlib.pyplot as plt
import numpy as np

from pypetal.utils.petalio import err2str

mpl.rcParams['xtick.minor.visible'] = True
mpl.rcParams['xtick.top'] = True
mpl.rcParams['xtick.direction'] = 'in'

mpl.rcParams['ytick.minor.visible'] = True
mpl.rcParams['ytick.right'] = True
mpl.rcParams['ytick.direction'] = 'in'

mpl.rcParams["figure.autolayout"] = False

mpl.rcParams['savefig.dpi'] = 300
mpl.rcParams['savefig.format'] = 'pdf'




def plot_pyccf_results(x1, y1, yerr1, x2, y2, yerr2,
                       ccf_lags, ccf,
                       cccd_lags, ccpd_lags,
                       nbin=50, time_unit='d', lc_unit=['mJy', 'mJy'], lc_names=['', ''],
                       fname=None, show=False):

    """Plot the results of using pyCCF on two light curves.

    Parameters
    ----------

    x1 : array-like
        Time array for first light curve.

    y1 : array-like
        Values for the first light curve.

    yerr1 : array-like
        Uncertainty in the first light curve.

    x2 : array-like
        Time array for second light curve.

    y2 : array-like
        Values for the second light curve.

    yerr2 : array-like
        Uncertainty in the second light curve.

    ccf_lags : array-like
        Lags for the CCF.

    ccf : array-like
        CCF values.

    cccd_lags : array-like
        The output simulated lags from the CCCD.

    ccpd_lags : array-like
        The output simulated lags from the CCPD.

    nbin : int, optional
        Number of bins to use for the histograms. Default is 50.

    time_unit : str, optional
        The unit of time for the light curves for the plot. Default is 'd'.

    lc_unit : str, optional
        The unit of the light curves for the plot. Default is 'mJy'.

    lc_names : list, optional
        The names of the light curves to use for the plot. Default is ['', ''].

    fname : str, optional
        If not ``None``, will save the plot to the given filename. Default is ``None``.

    show : bool, optional
        If ``True``, will show the plot. Default is ``False``.

    """


    cent_med = np.median( cccd_lags )
    cent_hi = np.percentile( cccd_lags, 84 )
    cent_lo = np.percentile( cccd_lags, 16 )

    peak_med = np.median( ccpd_lags )
    peak_hi = np.percentile( ccpd_lags, 84 )
    peak_lo = np.percentile( ccpd_lags, 16 )

    min_lag = np.min(ccf_lags)
    max_lag = np.max(ccf_lags)


    lag_unit = time_unit



    #Setup plot axes
    fig = plt.figure( figsize=(12, 7) )
    gs = gridspec.GridSpec(ncols=3, nrows=6)

    ax1 = fig.add_subplot(gs[:3, :2])
    ax2 = fig.add_subplot(gs[3:6, :2], sharex=ax1)

    ax3 = fig.add_subplot(gs[:2, 2])
    ax4 = fig.add_subplot(gs[2:4, 2], sharex=ax3)
    ax5 = fig.add_subplot(gs[4:6, 2], sharex=ax3)



    #Plot light curve data
    _, caps, bars = ax1.errorbar(x1, y1, yerr1, fmt='.k')
    [bar.set_alpha(.25) for bar in bars]

    _, _, bars = ax2.errorbar(x2, y2, yerr2, fmt='.k')
    [bar.set_alpha(.25) for bar in bars]


    ax2.set_xlabel('Time [' + str(time_unit) + ']', fontsize=19)

    #----------------------------
    #Plot CCF
    ax3.plot( ccf_lags, ccf )
    ax3.set_ylim(-1, 1)

    #----------------------------
    #Plot CCCD
    bin_vals, bin_edges, _ = ax4.hist( cccd_lags, nbin, range=(min_lag, max_lag), density=False )

    #Increase y-bounds to fit text
    ymin, ymax = ax4.get_ylim()
    ymax = ymin + (ymax - ymin)*1.1
    ax4.set_ylim(ymin, ymax)

    res_txt = r'$' + err2str(cent_med, cent_hi-cent_med, cent_med-cent_lo, dec=2) + r'$ ' + lag_unit
    ax4.text(.05, .8, res_txt, transform=ax4.transAxes, fontsize=14)

    #----------------------------
    #Plot CCPD
    bin_vals, _, _ = ax5.hist( ccpd_lags, bins=bin_edges, range=(min_lag, max_lag), density=False )

    #Increase y-bounds to fit text
    ymin, ymax = ax5.get_ylim()
    ymax = ymin + (ymax - ymin)*1.1
    ax5.set_ylim(ymin, ymax)


    res_txt = r'$' + err2str( peak_med, peak_hi-peak_med, peak_med-peak_lo, dec=2 ) + r'$ ' + lag_unit
    ax5.text(.05, .8, res_txt, transform=ax5.transAxes, fontsize=14)
    ax5.set_xlabel('Lag [' + str(lag_unit) + ']', fontsize=19)


    #Add ylabels
    if lc_unit[0] == 'mag':
        ytxt = 'Magnitude'
        ax1.invert_yaxis()

    elif (lc_unit[0] == 'Arbitrary Units') or (lc_unit[0] == ''):
        ytxt = 'Flux'
    else:
        ytxt = 'Flux [' + str(lc_unit[0]) + ']'
    ax1.text( -.2, .5, ytxt, transform=ax1.transAxes, rotation='vertical', ha='left', va='center', fontsize=19 )


    if lc_unit[1] == 'mag':
        ytxt = 'Magnitude'
        ax2.invert_yaxis()

    elif (lc_unit[1] == 'Arbitrary Units') or (lc_unit[1] == ''):
        ytxt = 'Flux'
    else:
        ytxt = 'Flux [' + str(lc_unit[1]) + ']'
    ax2.text( -.2, .5, ytxt, transform=ax2.transAxes, rotation='vertical', ha='left', va='center', fontsize=19 )


    #Increase y-bounds to fit text
    ymin, ymax = ax1.get_ylim()
    ymax = ymin + (ymax - ymin)*1.15
    ax1.set_ylim(ymin, ymax)

    ymin, ymax = ax2.get_ylim()
    ymax = ymin + (ymax - ymin)*1.15
    ax2.set_ylim(ymin, ymax)

    ax1.text( .05, .95, lc_names[0], transform=ax1.transAxes, ha='left', va='top', fontsize=17 )
    ax2.text( .05, .95, lc_names[1], transform=ax2.transAxes, ha='left', va='top', fontsize=17 )



    plt.figtext( .94, .72, 'CCF', rotation=270, fontsize=19 )
    plt.figtext( .94, .45, 'CCCD', rotation=270, fontsize=19 )
    plt.figtext( .94, .2, 'CCPD', rotation=270, fontsize=19 )


    for i, ax in enumerate([ax3, ax4, ax5]):
        ax.yaxis.tick_right()
        ax.yaxis.set_ticks_position('both')

        if i == 2:
            ax.tick_params('both', labelsize=12)
        else:
            ax.tick_params('x', labelsize=0)
            ax.tick_params('y', labelsize=12)

        ax.tick_params('both', which='major', length=6)
        ax.tick_params('both', which='minor', length=3)

        ax.locator_params('x', nbins=5)
        ax.locator_params('y', nbins=3)


    for i, ax in enumerate([ax1, ax2]):

        if i == 1:
            ax.tick_params('both', labelsize=13)
        else:
            ax.tick_params('x', labelsize=0)
            ax.tick_params('y', labelsize=13)

        ax.tick_params('both', which='major', length=7)
        ax.tick_params('both', which='minor', length=3)

        ax.locator_params('y', nbins=6)


    plt.subplots_adjust(hspace=.07, wspace=.1)

    if fname is not None:
        plt.savefig(fname, dpi=300, bbox_inches='tight')

    if show:
        plt.show()

    plt.cla()
    plt.clf()
    plt.close()

    del fig, ax1, ax2, ax3, ax4, ax5, gs
    return



import glob
import os

import numpy as np
from astropy.table import Table

import pypetal.utils.detrending as dtr
from pypetal.drw_rej.module import drw_rej_tot
from pypetal.pyccf.module import pyccf_tot
from pypetal.pyroa.module import pyroa_tot
from pypetal.pyzdcf.module import pyzdcf_tot
from pypetal.utils import defaults
from pypetal.utils.petalio import make_directories, write_data
from pypetal.weighting.module import run_weighting_tot


def run_pipeline(output_dir, arg2,
                 line_names=None,
                 run_drw_rej=False, drw_rej_params={},
                 run_detrend=False, detrend_params={},
                 run_pyccf=False, pyccf_params={},
                 run_pyzdcf=False, pyzdcf_params={},
                 run_pyroa=False, pyroa_params={},
                 **kwargs):

    """ Run the pyPetal pipeline on a list of files. Individual modules can be specified, but are not run by default.

    Parameters
    ----------

    output_dir : str
        The directory to save the output files to.

    arg2 : list of str, list of floats
        If a list of strings, the filenames of the light curves to run the pipeline on. If a list of floats, the light curves themselves.

    line_names : list of str, optional
        The names of the lines to be used in the output files. If ``None``, the lines will be named as ``line1``, ``line2``,... Default is ``None``.

    run_drw_rej : bool, optional
        Whether to run the DRW rejection module. Default is ``False``.

    drw_rej_params : dict, optional
        The parameters to pass to the DRW rejection module. Default is ``{}``.

    run_detrend : bool, optional
        Whether to run the detrending module. Default is ``False``.

    detrend_params : dict, optional
        The parameters to pass to the detrending module. Default is ``{}``.

    run_pyccf : bool, optional
        Whether to run the pyCCF module. Default is ``False``.

    pyccf_params : dict, optional
        The parameters to pass to the pyCCF module. Default is ``{}``.

    run_pyzdcf : bool, optional
        Whether to run the pyZDCF module. Default is ``False``.

    pyzdcf_params : dict, optional
        The parameters to pass to the pyZDCF module. Default is ``{}``.

    run_pyroa : bool, optional
        Whether to run the PyROA module. Default is ``False``.

    pyroa_params : dict, optional
        The parameters to pass to the PyROA module. Default is ``{}``.




    Returns
    -------

    output : dict
        A dictionary containing the output of the pipeline modules.

    """

    output_dir = os.path.abspath(output_dir) + r'/'

    if arg2 is None:
        raise Exception('Please provide a list of light curve filenames or the light curves themselves')


    if not isinstance(arg2[0], str):
        os.makedirs( output_dir + 'input_lcs/', exist_ok=True )
        fnames = []

        for i in range( len(arg2) ):

            if i == 0:
                name = 'continuum'
            else:
                name = 'line{}'.format(i+1)

            write_data( arg2[i], output_dir + 'input_lcs/' + name + '.dat' )
            fnames.append( output_dir + 'input_lcs/' + name + '.dat' )

        fnames = np.array(fnames)
        kwargs['file_fmt'] = 'csv'
    else:
        fnames = arg2




    if len(fnames) < 2:
        print('ERROR: Requires at least two light curves to run pipeline.')
        return {}

    if len(line_names) != len(fnames):
        print('ERROR: Must have the same number of line names as light curves.')
        return {}


    cont_fname = fnames[0]
    line_fnames = fnames[1:]

    if type(line_fnames) is str:
        line_fnames = [line_fnames]


    #Read in general kwargs
    general_kwargs = defaults.set_general(kwargs, fnames)

    #Get "reject_data"
    _, _, _, _, _, _, reject_data, _ = defaults.set_drw_rej(drw_rej_params, fnames)

    #Get "together_pyroa"
    _, _, _, _, _, _, _, _, together_pyroa, _ = defaults.set_pyroa(pyroa_params, len(fnames))


    #Name lines if unnamed
    if line_names is None:
        line_names = np.zeros( len(line_fnames), dtype=str )
        for i in range(len(line_fnames)):
            line_names.append('Line {}'.format(i+1))


    #Get directory common to all files
    input_dir = os.path.dirname( os.path.realpath(cont_fname) )
    for i in range(len(line_fnames)):
        assert input_dir == os.path.dirname( os.path.realpath(line_fnames[i]) )



    #Create subdirectories for each line and module
    make_directories(output_dir, fnames, line_names,
                     run_drw_rej, run_pyccf, run_pyzdcf, run_pyroa,
                     reject_data, together_pyroa)

    if general_kwargs['file_fmt'] != 'csv':
        #Make temp directory to store csv files
        os.makedirs( output_dir + 'temp/', exist_ok=True )
        input_dir = output_dir + 'temp/'

        #Read in file
        try:
            dat = Table.read( cont_fname, format=general_kwargs['file_fmt'])
        except:
            dat = Table.read( cont_fname, format='ascii')


        #Get filename
        fname = os.path.basename( cont_fname )

        #Write to csv
        write_data( [ dat[ dat.colnames[0] ], dat[ dat.colnames[1] ], dat[ dat.colnames[2] ] ], input_dir + fname )
        cont_fname = input_dir + fname


        for i in range(len(line_fnames)):

            #Read in file
            try:
                dat = Table.read( line_fnames[i], format=general_kwargs['file_fmt'])
            except:
                dat = Table.read( line_fnames[i], format='ascii')

            #Get filename
            fname = os.path.basename( line_fnames[i] )

            #Write to csv
            write_data( [ dat[ dat.colnames[0] ], dat[ dat.colnames[1] ], dat[ dat.colnames[2] ] ], input_dir + fname )
            line_fnames[i] = input_dir + fname


    drw_rej_res = {}
    detrend_res = {}
    pyccf_res = {}
    pyzdcf_res = {}

    if run_drw_rej:

        if np.any( reject_data ):
            os.makedirs( output_dir + 'processed_lcs/', exist_ok=True )

        drw_rej_res = drw_rej_tot( cont_fname, line_fnames, line_names, output_dir, general_kwargs, drw_rej_params )

        #Output light curve files with masks
        for i in range(len(fnames)):

            if i == 0:
                x, y, yerr = np.loadtxt( cont_fname, delimiter=',', unpack=True, usecols=[0,1,2] )
            else:
                x, y, yerr = np.loadtxt( line_fnames[i-1], delimiter=',', unpack=True, usecols=[0,1,2] )

            mask = drw_rej_res['masks'][i]
            write_data( [x,y,yerr,mask], output_dir + 'light_curves/' + line_names[i] + '.dat', '#x,y,yerr,mask' )


        #If rejecting any data, make the new files the ones without rejected data
        if np.any( reject_data ):
            cont_fname = output_dir + 'processed_lcs/' + line_names[0] + '_data.dat'

            for i in range(len(line_fnames)):
                line_fnames[i] = output_dir + 'processed_lcs/' + line_names[i+1] + '_data.dat'


    else:

        #Output light curve files with masks
        for i in range(len(fnames)):

            if i == 0:
                x, y, yerr = np.loadtxt( cont_fname, delimiter=',', unpack=True, usecols=[0,1,2] )
            else:
                x, y, yerr = np.loadtxt( line_fnames[i-1], delimiter=',', unpack=True, usecols=[0,1,2] )

            write_data( [x,y,yerr,np.full_like(x, False, dtype=bool)], output_dir + 'light_curves/' + line_names[i] + '.dat' )


    if run_detrend:

        if not run_drw_rej:
            os.makedirs( output_dir + 'processed_lcs/', exist_ok=True )
        elif (run_drw_rej) & ( ~np.all(reject_data) ):
            os.makedirs( output_dir + 'processed_lcs/', exist_ok=True )

        detrend_res = dtr.detrend_tot(output_dir, cont_fname, line_fnames, line_names, general_kwargs, detrend_params)

        if not run_drw_rej:
            cont_fname = output_dir + 'processed_lcs/' + line_names[0] + '_data.dat'

            for i in range(len(line_fnames)):
                line_fnames[i] = output_dir + 'processed_lcs/' + line_names[i+1] + '_data.dat'

    if run_pyccf:
        pyccf_res = pyccf_tot(cont_fname, line_fnames, line_names, output_dir, general_kwargs, pyccf_params)

    if run_pyzdcf:
        pyzdcf_res, plike_res = pyzdcf_tot(cont_fname, line_fnames, line_names, output_dir, general_kwargs, pyzdcf_params)

    if run_pyroa:
        pyroa_res = pyroa_tot(cont_fname, line_fnames, line_names, output_dir, general_kwargs, pyroa_params)


    if general_kwargs['file_fmt'] != 'csv':
        import shutil
        shutil.rmtree(input_dir)


    #Compile all results into a single dict
    tot_res = {}

    if run_drw_rej:
        tot_res['drw_rej_res'] = drw_rej_res

    if run_detrend:
        tot_res['detrend_res'] = detrend_res

    if run_pyccf:
        tot_res['pyccf_res'] = pyccf_res

    if run_pyzdcf:
        tot_res['pyzdcf_res'] = pyzdcf_res
        tot_res['plike_res'] = plike_res

    if run_pyroa:
        tot_res['pyroa_res'] = pyroa_res



    if not isinstance(arg2[0], str):
        import shutil
        shutil.rmtree( output_dir + 'input_lcs/' )


    return tot_res






def run_weighting(output_dir, line_names,
                 run_pyccf=False, pyccf_params={},
                 run_pyroa=False, pyroa_params={},
                 run_javelin=False, javelin_params={},
                 weighting_params={},
                 **kwargs):


    """Perform weighting on the results from the pyPetal pipeline.
    NOTE: This requires that you have run the pyPetal pipeline (i.e. pypetal.pipeline.run.pipeline) with
    at least run_pyccf=True.
    This will perform the weighting scheme described in Grier et al. (2019) to the lag distributions from
    each of the modules run (e.g., pyCCF, PyROA, JAVELIN).


    Parameters
    ----------
    output_dir : str
        The directory where the output files will be saved. Must be the same output directory as used
        in pyPetal and pyPetal-jav.

    line_names : list of str
        The names of the lines to be weighted. Must be the same as the line names used in pyPetal and
        pyPetal-jav.

    run_pyccf : bool, optional
        Whether or not pyCCF was run in the pyPetal pipeline. Default is False.

    pyccf_params : dict, optional
        The parameters used in the pyPetal pipeline for pyCCF. Default is {}.

    run_pyroa : bool, optional
        Whether or not PyROA was run in the pyPetal pipeline. Default is False.

    pyroa_params : dict, optional
        The parameters used in the pyPetal pipeline for PyROA. Default is {}.

    run_javelin : bool, optional
        Whether or not the pyPetal-jav pipeline was run. Default is False.

    javelin_params : dict, optional
        The parameters used in the pyPetal-jav pipeline. Default is {}.

    weighting_params : dict, optional
        The parameters to be used for the weighting. Default is {}.



    Returns
    -------

    res : dict
        A dictionary containing the results of the weighting.

    """


    if (not run_pyccf) & (not run_javelin) & (not run_pyroa):
        raise Exception('ERROR: Either JAVELIN, pyCCF, or PyROA must be run before weighting can be done.')

    output_dir = os.path.abspath(output_dir) + r'/'

    if output_dir + 'processed_lcs/' in glob.glob( output_dir + '*/' ):
        fnames = [ output_dir + 'processed_lcs/' + x + '_data.dat' for x in line_names ]
    else:
        fnames = [ output_dir + 'light_curves/' + x + '.dat' for x in line_names ]

    #Read in general kwargs
    general_kwargs = defaults.set_general(kwargs, fnames)

    #Get "interp"
    interp, _, _, _, _, _ = defaults.set_pyccf(pyccf_params)

    #Get "together" for javelin
    if 'together' in javelin_params:
        together_jav = javelin_params['together']
    else:
        together_jav = False

    #Get "together" for pyroa
    _, _, _, _, _, _, _, _, together_pyroa, _ = defaults.set_pyroa( pyroa_params, len(line_names) )


    javelin_chain_fnames = None
    pyccf_iccf_fnames = None
    pyccf_dist_fnames = None
    pyroa_sample_fnames = None

    if run_pyccf:
        pyccf_iccf_fnames = []
        pyccf_dist_fnames = []

        for x in line_names[1:]:
            pyccf_iccf_fnames.append( output_dir + x + '/pyccf/' + x + '_ccf.dat' )
            pyccf_dist_fnames.append( output_dir + x + '/pyccf/' + x + '_ccf_dists.dat' )


    if run_javelin:
        if together_jav:
            javelin_chain_fnames = output_dir + 'javelin/chain_rmap.txt'
        else:
            javelin_chain_fnames = []

            for x in line_names[1:]:
                javelin_chain_fnames.append( output_dir + x + '/javelin/chain_rmap.txt' )



    if run_pyroa:
        if together_pyroa:
            pyroa_sample_fnames = output_dir + 'pyroa/samples.obj'
        else:
            pyroa_sample_fnames = []

            for x in line_names[1:]:
                pyroa_sample_fnames.append( output_dir + x + '/pyroa/samples.obj' )


    res = run_weighting_tot(output_dir,
                                      javelin_chain_fnames, pyccf_iccf_fnames, pyccf_dist_fnames, pyroa_sample_fnames,
                                      line_names, interp, together_jav,
                                      pyroa_params=pyroa_params, general_kwargs=general_kwargs,
                                      weighting_params=weighting_params)

    return res



import pickle

import numpy as np
from scipy.signal import peak_widths

from pypetal.pyccf.utils import peakcent
from pypetal.pyroa.utils import get_samples_chunks
from pypetal.utils import defaults
from pypetal.utils.petalio import write_data, write_weighting_summary

###########################################################################
########################## ASSIST FUNCTIONS ###############################
###########################################################################

def find_overlap(x1, x2, gaps):

    """Find the amount of overlapping data points between two light curves,
    taking into account seasonal gaps in the data.

    Parameters
    ----------

    x1 : list of float
        The time values of the first light curve.

    x2 : list of float
        The time values of the second light curve.

    gaps : list of float
        A list of gaps in the data for the first light curve. The list should be structured
        a a list of 2 element arrays, where the first element is the start of the gap and the
        second element is the end of the gap.




    Returns
    -------

    Ntot : int
        The number of overlapping data points between the two light curves.

    """

    sort_ind = np.argsort(x1)
    x1 = x1[sort_ind]

    sort_ind = np.argsort(x2)
    x2 = x2[sort_ind]

    low_lim = np.maximum( x1[0], x2[0] )
    up_lim = np.minimum( x1[-1], x2[-1] )

    Ntot = len( np.argwhere( (x2 >= low_lim) & (x2 <= up_lim) ).T[0] )

    #Remove points in x2 that fall into gaps in x1
    if gaps is not None:
        for i in range(len(gaps[0])):
            lgap = gaps[0][i]
            rgap = gaps[1][i]

            bad_ind = np.argwhere( (x2 > lgap) & (x2 < rgap) ).T[0]
            Ntot -= len(bad_ind)

    return Ntot



def prob_tau(x1, x2, laglim=None, lagvals=None, Nlag=1000, gap_size=30, k=2):


    """Calculate the probability distribution P(tau) to weight the time lag distribution described in
    Grier et al. (2019).



    Parameters
    ----------

    x1 : list of float
        The time values of the first light curve.

    x2 : list of float
        The time values of the second light curve.

    laglim : list of float, optional
        The limits of lags to search over. If ``None``, the limits will be set to the baseline of the
        light curves. If not ``None``, Nlag must be set. Default is ``None``.

    lagvals : list of float, optional
        The values of lags to search over. If ``None``, the limits will be set to the baseline of the
        light curves. If not ``None``, Nlag must be set. Default is ``None``.

    Nlag : int, optional
        The number of lags to search over. Default is 1000.

    gap_size : float, optional
        The minimum gap size to use when searching for seasonal gaps in the first light curve.
        Default is 30.

    k : int, optional
        The power to raise the probability distribution to. Default is 2.




    Returns
    -------

    probs : list of float
        The probability distribution P(tau).

    nvals : list of float
        The number of overlapping data points for each lag N(tau).

    N0 : int
        The number of overlapping points when tau = 0.

    lags : list of float
        The lags used to calculate the probability distribution.

    """

    sort_ind = np.argsort(x1)
    x1 = x1[sort_ind]

    sort_ind = np.argsort(x2)
    x2 = x2[sort_ind]

    baseline = np.max([ x1[-1]-x1[0], x2[-1]-x2[0] ])
    Nlag = int(Nlag)

    if (laglim is None) & (lagvals is None):
        laglim = [-baseline, baseline]
        lags = np.linspace(laglim[0], laglim[1], Nlag)

    elif (lagvals is None) & (laglim is not None):
        lags = np.linspace(laglim[0], laglim[1], Nlag)

    else:
        lags = lagvals


    #Get where gaps are in data
    gap_ind = np.argwhere(np.diff(x1) > gap_size).T[0]
    gaps_left = []
    gaps_right = []

    for i in range(len(gap_ind)):
        gaps_left.append( x1[gap_ind[i]] )
        gaps_right.append( x1[gap_ind[i]+1] )

    gaps = np.array([ gaps_left, gaps_right ])

    if len(gaps_left) == 0:
        gaps = None

    nvals = np.zeros_like(lags)
    N0 = find_overlap( x1, x2, gaps)

    for i, tau in enumerate(lags):
        nvals[i] = find_overlap( x1, x2-tau, gaps )

    probs = (nvals / N0)**k

    return probs, nvals, N0, lags



def get_acf(x, y, interp=None, lag_bounds=None,
            sigmode=0.2, thres=0.8):


    """Calculate the autocorrelation function of a light curve.


    Parameters
    ----------

    x : list of float
        The time values of the light curve.

    y : list of float
        The flux values of the light curve.

    interp : float, optional
        The interpolation distance to use for pyCCF. Default is ``None``.

    lag_bounds : list of float, optional
        The limits of lags to search over. If ``None``, the limits will be set to the baseline of the
        light curve. Default is ``None``.

    sigmode : float, optional
        The ``sigmode`` option for pyCCF. Default is 0.2.

    thres : float, optional
        The ``thres`` option for pyCCF. Default is 0.8.


    Returns
    -------

    acf : list of float
        The autocorrelation function of the light curve.

    lags : list of float
        The lags used to calculate the ACF.

    """


    sort_ind = np.argsort(x)
    x = x[sort_ind]
    y = y[sort_ind]

    mean_diff = np.mean(np.diff(x))
    baseline = x[-1] - x[0]

    if lag_bounds is None:
        lag_bounds = [-baseline, baseline]

    if interp is None:
        interp = mean_diff/2

    _, _, _, _, ccf_pack, \
        _, _, _ = peakcent(x, y, x, y, lag_bounds[0], lag_bounds[1], interp,
                                 thres=thres, sigmode=sigmode)

    return ccf_pack[0], ccf_pack[1]





def get_weights(x1, y1, x2, y2, interp=None, lag_bounds=None,
                sigmode=0.2, thres=0.8, gap_size=30, k=2):


    """Calculate the total weighting distribution w(tau) described in Grier et al. (2019).
    This calculates the probability weights depending on the number of overlapping data points as
    a function of time lag P(tau) and convolves it with the ACF of the continuum light curve.


    Parameters
    ----------

    x1 : list of float
        The time values of the first light curve.

    y1 : list of float
        The flux values of the first light curve.

    x2 : list of float
        The time values of the second light curve.

    y2 : list of float
        The flux values of the second light curve.

    interp : float, optional
        The interpolation distance to use for pyCCF. Default is ``None``.

    lag_bounds : list of float, optional
        The limits of lags to search over. If ``None``, the limits will be set to the baseline of the
        light curve. Default is ``None``.

    sigmode : float, optional
        The ``sigmode`` option for pyCCF. Default is 0.2.

    thres : float, optional
        The ``thres`` option for pyCCF. Default is 0.8.

    gap_size : float, optional
        The minimum gap size to use when searching for seasonal gaps in the first light curve.
        Default is 30.

    k : int, optional
        The power to raise the probability distribution to. Default is 2.



    Returns
    -------

    prob_dist : list of float
        The total weighting distribution w(tau).

    lags : list of float
        The lags used to calculate the total weighting distribution.

    ntau : list of float
        The number of overlapping data points as a function of time lag N(tau).

    acf : list of float
        The ACF of the first light curve.

    n0 : float
        The number of overlapping data points at a lag tau = 0.

    """

    #Get ACF for continuum light curve
    acf, acf_lags = get_acf(x1, y1, interp=interp, lag_bounds=lag_bounds,
                            sigmode=sigmode, thres=thres)


    #Set all values past first negative to zero

        #Left side
    left_inds = np.argwhere( (acf < 0) & (acf_lags < 0) ).T[0]
    if len(left_inds) == 0:
        ind1 = 0
    else:
        ind1 = np.max( left_inds )

        #Right side
    right_inds = np.argwhere( (acf < 0) & (acf_lags > 0) ).T[0]
    if len(right_inds) == 0:
        ind2 = len(acf)-1
    else:
        ind2 = np.min( np.argwhere( (acf < 0) & (acf_lags > 0) ).T[0] )


    acf_new = acf.copy()
    acf_new[:ind1+1] = 0
    acf_new[ind2:] = 0

    #Get p(tau)
    probs, ntau, n0, lags = prob_tau(x1, x2, lagvals=acf_lags, gap_size=gap_size, k=k)

    #Convolve ACF with p(tau)
    tot_weight = np.convolve(acf_new, probs, mode='same')
    prob_dist = tot_weight / np.max(tot_weight)

    return prob_dist, lags, ntau, acf, n0




def gaussian(x, sigma):
    c = np.sqrt(2 * np.pi) * sigma
    return np.exp(-0.5 * (x / sigma)**2) / c

def get_bounds(dist, weights, lags, width=15, rel_height=.99):


    """Find the bounds of the peak of the weighted distribution, given the original distribution and weights.
    This will convolve this weighted distribution with a Gaussian and find the bounds of the primary (i.e tallest)
    peak in the distribution.


    Parameters
    ----------

    dist : list of float
        The original distribution.

    weights : list of float
        The weights to apply to the distribution.

    lags : list of float
        The lags used to calculate the distribution.

    width : float, optional
        The width of the Gaussian to convolve the distribution with. Default is 15.



    Returns
    -------

    peak_lower_bound : float
        The lag of the lower bound of the peak of the distribution.

    peak: int
        The lag of the peak of the distribution.

    peak_upper_bound : float
        The lag of the upper bound of the peak of the distribution.

    smooth_dist : list of float
        The smoothed original distribution.

    smooth_weight_dist : list of float
        The smoothed weighted distribution.

    """

    #Bin dist into weight lags
    dbin = lags[1] - lags[0]
    bin0 = np.min(lags)

    bin_edges = []
    bin0 = np.min(lags) - dbin/2

    for i in range(len(lags)+1):
        bin_edges.append( bin0 + i*dbin )

    hist, _ = np.histogram(dist, bins=bin_edges)
    hist = np.array(hist, dtype=float)

    #Weight histogram
    weighted_hist = hist*weights

    #Convolve weighted distribution with gaussian
    gauss = gaussian( lags, width )
    smooth_dist = np.convolve( hist, gauss, mode='same')
    smooth_weight_dist = np.convolve( weighted_hist, gauss, mode='same')

    #Find peak
    peak_ind = np.argmax(smooth_weight_dist)

    #Find peak bounds
    res = peak_widths( smooth_weight_dist, [peak_ind], rel_height=rel_height )

    peak = lags[peak_ind]
    bound_left = lags[ np.floor(res[2]).astype(int) ] + dbin*( res[2]%1 )
    bound_right = lags[ np.floor(res[3]).astype(int) ] + dbin*( res[3]%1 )

    return bound_left[0], peak, bound_right[0], smooth_dist, smooth_weight_dist

###########################################################################
###########################################################################
###########################################################################

def combine_weight_outputs(res_arr, run_pyccf, run_javelin, run_pyroa):

    output = {
        'pyccf': {},
        'javelin': {},
        'pyroa': {},
        'rmax_pyccf': [],
        'rmax_javelin': [],
        'rmax_pyroa': []
    }

    cols_pyccf = ['centroid']
    cols_javelin = ['tophat_lag']
    cols_pyroa = ['time_delay']

    cols_common = ['bounds', 'acf', 'lags', 'weight_dist', 'smoothed_dist', 'ntau', 'frac_rejected']
    for col in cols_common:
        cols_pyccf.append(col)
        cols_javelin.append(col)
        cols_pyroa.append(col)

    cols_pyccf.append('CCCD')
    cols_javelin.append('lag_dist')
    cols_pyroa.append('lag_dist')

    cols_pyccf.append('downsampled_CCCD')
    cols_javelin.append('downsampled_lag_dist')
    cols_pyroa.append('downsampled_lag_dist')


    run_arr = [run_pyccf, run_javelin, run_pyroa]
    cols_arr = [cols_pyccf, cols_javelin, cols_pyroa]
    names_arr = ['pyccf', 'javelin', 'pyroa']

    for run, cols, name in zip(run_arr, cols_arr, names_arr):
        if run:
            for col in cols:
                output[name][col] = []


    for res in res_arr:
        for run, cols, name in zip(run_arr, cols_arr, names_arr):
            if run:
                for col in cols:
                    output[name][col].append( res[name][col] )


        #rmax
        if run_pyccf:
            output['rmax_pyccf'].append( res['rmax_pyccf'] )

            if run_javelin:
                output['rmax_javelin'].append( res['rmax_javelin'] )
            if run_pyroa:
                output['rmax_pyroa'].append( res['rmax_pyroa'] )

    return output



def run_weighting_single( output_dir, cont_fname, line_fname,
                          weighting_kwargs, lag_bounds='baseline',
                          jav_chain_file=None,
                          pyccf_iccf_file=None, pyccf_dist_file=None,
                          pyroa_sample_file=None,
                          javelin_lag_col=2, pyroa_obj_ind=1,
                          pyccf_params={}, pyroa_params = {},
                          plot=False, time_unit='d'):

    #---------------------------
    # pyCCF kwargs
    interp, _, _, sigmode, thres, _ = defaults.set_pyccf(pyccf_params)

    #---------------------------
    #Weighting kwargs
    gap_size, k, width, rel_height, _ = defaults.set_weighting(weighting_kwargs)

    #---------------------------
    #See what modules to run

    if jav_chain_file is not None:
        run_javelin = True
    else:
        run_javelin = False

    if pyccf_dist_file is not None:
        run_pyccf = True
    else:
        run_pyccf = False

    if pyroa_sample_file is not None:
        run_pyroa = True
    else:
        run_pyroa = False


    cols = ['n0', 'peak_bounds', 'peak', 'lag', 'lag_err', 'frac_rejected']
    modules = ['pyccf', 'javelin', 'pyroa']

    summary_dict = {}
    summary_dict['k'] = k

    for mod in modules:
        for col in cols:
            summary_dict[ col + '_' + mod ] = None

        summary_dict['rmax_' + mod] = None


    output = {}
    for mod in modules:
        output[mod] = {}
        output['rmax_' + mod] = []


    #---------------------------
    #Get continuum data
    x_cont, y_cont, _ = np.loadtxt(cont_fname, usecols=[0,1,2], unpack=True, delimiter=',')
    x_line, y_line, _ = np.loadtxt(line_fname, unpack=True, delimiter=',', usecols=[0,1,2])

    if lag_bounds == 'baseline':
        baseline = np.max([ x_cont.max(), x_line.max() ]) - np.min([ x_cont.min(), x_line.min() ])
        lag_bounds = [-baseline, baseline]

    #---------------------------
    #pyCCF

    if run_pyccf:
        cccd_lags, ccpd_lags = np.loadtxt(pyccf_dist_file, unpack=True, delimiter=',')
        prob_dist, lags, ntau, acf, n0 = get_weights(x_cont, y_cont, x_line, y_line,
                                    interp=interp, lag_bounds=lag_bounds,
                                    sigmode=sigmode, thres=thres,
                                    gap_size=gap_size, k=k)

        min_bound, peak, max_bound, smooth_dist, smooth_weight_dist = get_bounds(cccd_lags, prob_dist, lags, width=width, rel_height=rel_height)
        downsampled_cccd = cccd_lags[(cccd_lags > min_bound) & (cccd_lags < max_bound)]

        med_cent = np.median(downsampled_cccd)
        cent_err_lo = med_cent - np.percentile( downsampled_cccd, 16 )
        cent_err_hi = np.percentile( downsampled_cccd, 84 ) - med_cent



        #Write diagnostic info
        write_data( [ lags, ntau, prob_dist, acf, smooth_dist, smooth_weight_dist ],
                    output_dir + 'pyccf_weights.dat',
                    '#lags,ntau,weight_dist,acf,smooth_dist,smooth_weight_dist')

        write_data( downsampled_cccd,
                    output_dir + 'pyccf_weighted_cccd.dat')


        #Add to summary dict
        summary_dict['n0_pyccf'] = n0
        summary_dict['peak_bounds_pyccf'] = [min_bound, max_bound]
        summary_dict['peak_pyccf'] = peak
        summary_dict['lag_pyccf'] = med_cent
        summary_dict['lag_err_pyccf'] = [cent_err_lo, cent_err_hi]
        summary_dict['frac_rejected_pyccf'] = 1 - len(downsampled_cccd) / len(cccd_lags)


        #Add to weighting results
        output['pyccf']['centroid'] = [cent_err_lo, med_cent, cent_err_hi]
        output['pyccf']['bounds'] = [min_bound, peak, max_bound]
        output['pyccf']['acf'] = acf
        output['pyccf']['lags'] = lags
        output['pyccf']['weight_dist'] = prob_dist
        output['pyccf']['smoothed_dist'] = smooth_weight_dist
        output['pyccf']['ntau'] = ntau
        output['pyccf']['CCCD'] = cccd_lags
        output['pyccf']['downsampled_CCCD'] = downsampled_cccd
        output['pyccf']['frac_rejected'] = 1 - len(downsampled_cccd) / len(cccd_lags)

    else:
        #Add to summary dict
        summary_dict['n0_pyccf'] = np.nan
        summary_dict['peak_bounds_pyccf'] = [np.nan, np.nan]
        summary_dict['peak_pyccf'] = np.nan
        summary_dict['lag_pyccf'] = np.nan
        summary_dict['lag_err_pyccf'] = [np.nan, np.nan]
        summary_dict['frac_rejected_pyccf'] = np.nan


        #Add to weighting results
        output['pyccf']['centroid'] =  [np.nan, np.nan, np.nan]
        output['pyccf']['bounds'] =  [np.nan, np.nan, np.nan]
        output['pyccf']['acf'] = np.nan
        output['pyccf']['lags'] = np.nan
        output['pyccf']['weight_dist'] = np.nan
        output['pyccf']['smoothed_dist'] = np.nan
        output['pyccf']['ntau'] = np.nan
        output['pyccf']['CCCD'] = np.nan
        output['pyccf']['downsampled_CCCD'] = np.nan
        output['pyccf']['frac_rejected'] = np.nan


    #---------------------------
    #JAVELIN

    if run_javelin:
        jav_chains = np.loadtxt(jav_chain_file, unpack=True)
        lag_dist = jav_chains[javelin_lag_col]
        prob_dist, lags, ntau, acf, n0 = get_weights(x_cont, y_cont, x_line, y_line,
                                            interp=interp, lag_bounds=lag_bounds,
                                            sigmode=sigmode, thres=thres,
                                            gap_size=gap_size, k=k)

        min_bound, peak, max_bound, smooth_dist, smooth_weight_dist = get_bounds(lag_dist, prob_dist, lags, width=width)
        downsampled_dist = lag_dist[(lag_dist > min_bound) & (lag_dist < max_bound)]

        med_lag = np.median(downsampled_dist)
        lag_err_lo = med_lag - np.percentile( downsampled_dist, 16 )
        lag_err_hi = np.percentile( downsampled_dist, 84 ) - med_lag


        #Write diagnostic info
        write_data( [ lags, ntau, prob_dist, acf, smooth_dist, smooth_weight_dist ],
                    output_dir + 'javelin_weights.dat',
                    '#lags,ntau,weight_dist,acf,smooth_dist,smooth_weight_dist')

        write_data( downsampled_dist,
                output_dir + 'javelin_weighted_lag_dist.dat')



        #Add to summary dict
        summary_dict['n0_javelin'] = n0
        summary_dict['peak_bounds_javelin'] = [min_bound, max_bound]
        summary_dict['peak_javelin'] = peak
        summary_dict['lag_javelin'] = med_lag
        summary_dict['lag_err_javelin'] = [lag_err_lo, lag_err_hi]
        summary_dict['frac_rejected_javelin'] = 1 - len(downsampled_dist) / len(lag_dist)


        #Add to weighting results
        output['javelin']['tophat_lag'] =  [lag_err_lo, med_lag, lag_err_hi]
        output['javelin']['bounds'] =  [min_bound, peak, max_bound]
        output['javelin']['acf'] = acf
        output['javelin']['lags'] = lags
        output['javelin']['weight_dist'] = prob_dist
        output['javelin']['smoothed_dist'] = smooth_weight_dist
        output['javelin']['ntau'] = ntau
        output['javelin']['lag_dist'] = lag_dist
        output['javelin']['downsampled_lag_dist'] = downsampled_dist
        output['javelin']['frac_rejected'] = 1 - len(downsampled_dist) / len(lag_dist)

    else:
        #Add to summary dict
        summary_dict['n0_javelin'] = np.nan
        summary_dict['peak_bounds_javelin'] = [np.nan, np.nan]
        summary_dict['peak_javelin'] = np.nan
        summary_dict['lag_javelin'] = np.nan
        summary_dict['lag_err_javelin'] = [np.nan, np.nan]
        summary_dict['frac_rejected_javelin'] = np.nan


        #Add to weighting results
        output['javelin']['tophat_lag'] =  [np.nan, np.nan, np.nan]
        output['javelin']['bounds'] =  [np.nan, np.nan, np.nan]
        output['javelin']['acf'] = np.nan
        output['javelin']['lags'] = np.nan
        output['javelin']['weight_dist'] = np.nan
        output['javelin']['smoothed_dist'] = np.nan
        output['javelin']['ntau'] = np.nan
        output['javelin']['lag_dist'] = np.nan
        output['javelin']['downsampled_lag_dist'] = np.nan
        output['javelin']['frac_rejected'] = np.nan


    #---------------------------
    #PyROA

    if run_pyroa:
        _, nburn, _, _, _, add_var, delay_dist, _, _, _ = defaults.set_pyroa(pyroa_params, 2)
        if isinstance(add_var, list):
            add_var = add_var[pyroa_obj_ind-1]
        if isinstance(delay_dist, list):
            delay_dist = delay_dist[pyroa_obj_ind-1]

        pyroa_samples = pickle.load(open(pyroa_sample_file, 'rb'))
        samples_chunks = get_samples_chunks( pyroa_samples, nburn, add_var, delay_dist )

        lag_dist = samples_chunks[pyroa_obj_ind][2]
        prob_dist, lags, ntau, acf, n0 = get_weights(x_cont, y_cont, x_line, y_line,
                                            interp=interp, lag_bounds=lag_bounds,
                                            sigmode=sigmode, thres=thres,
                                            gap_size=gap_size, k=k)

        min_bound, peak, max_bound, smooth_dist, smooth_weight_dist = get_bounds(lag_dist, prob_dist, lags, width=width)
        downsampled_dist = lag_dist[(lag_dist > min_bound) & (lag_dist < max_bound)]

        med_lag = np.median(downsampled_dist)
        lag_err_lo = med_lag - np.percentile( downsampled_dist, 16 )
        lag_err_hi = np.percentile( downsampled_dist, 84 ) - med_lag


        #Write diagnostic info
        write_data( [ lags, ntau, prob_dist, acf, smooth_dist, smooth_weight_dist ],
                    output_dir + 'pyroa_weights.dat',
                    '#lags,ntau,weight_dist,acf,smooth_dist,smooth_weight_dist')

        write_data( downsampled_dist,
                output_dir + 'pyroa_weighted_lag_dist.dat')



        #Add to summary dict
        summary_dict['n0_pyroa'] = n0
        summary_dict['peak_bounds_pyroa'] = [min_bound, max_bound]
        summary_dict['peak_pyroa'] = peak
        summary_dict['lag_pyroa'] = med_lag
        summary_dict['lag_err_pyroa'] = [lag_err_lo, lag_err_hi]
        summary_dict['frac_rejected_pyroa'] = 1 - len(downsampled_dist) / len(lag_dist)


        #Add to weighting results
        output['pyroa']['time_delay'] =  [lag_err_lo, med_lag, lag_err_hi]
        output['pyroa']['bounds'] =  [min_bound, peak, max_bound]
        output['pyroa']['acf'] = acf
        output['pyroa']['lags'] = lags
        output['pyroa']['weight_dist'] = prob_dist
        output['pyroa']['smoothed_dist'] = smooth_weight_dist
        output['pyroa']['ntau'] = ntau
        output['pyroa']['lag_dist'] = lag_dist
        output['pyroa']['downsampled_lag_dist'] = downsampled_dist
        output['pyroa']['frac_rejected'] = 1 - len(downsampled_dist) / len(lag_dist)

    else:
        #Add to summary dict
        summary_dict['n0_pyroa'] = np.nan
        summary_dict['peak_bounds_pyroa'] = [np.nan, np.nan]
        summary_dict['peak_pyroa'] = np.nan
        summary_dict['lag_pyroa'] = np.nan
        summary_dict['lag_err_pyroa'] = [np.nan, np.nan]
        summary_dict['frac_rejected_pyroa'] = np.nan


        #Add to weighting results
        output['pyroa']['time_delay'] =  [np.nan, np.nan, np.nan]
        output['pyroa']['bounds'] =  [np.nan, np.nan, np.nan]
        output['pyroa']['acf'] = np.nan
        output['pyroa']['lags'] = np.nan
        output['pyroa']['weight_dist'] = np.nan
        output['pyroa']['smoothed_dist'] = np.nan
        output['pyroa']['ntau'] = np.nan
        output['pyroa']['lag_dist'] = np.nan
        output['pyroa']['downsampled_lag_dist'] = np.nan
        output['pyroa']['frac_rejected'] = np.nan




    #---------------------------
    #Get rmax

    if run_pyccf:
        ccf_lags, ccf = np.loadtxt(pyccf_iccf_file, unpack=True, delimiter=',')


        #PyCCF lag
        lag = output['pyccf']['centroid'][1]
        lag_err_hi = output['pyccf']['centroid'][2]
        lag_err_lo = output['pyccf']['centroid'][0]
        good_ind = np.argwhere( ( ccf_lags >= lag-lag_err_lo ) & ( ccf_lags <= lag+lag_err_hi ) ).T[0]

        if len(good_ind) > 0:
            rmax_pyccf = np.max(ccf[good_ind])
        else:
            rmax_pyccf = ccf[ np.argmin( np.abs(lag - ccf_lags) ) ]

        summary_dict['rmax_pyccf'] = rmax_pyccf
        output['rmax_pyccf'] = rmax_pyccf



        if run_javelin:
            #JAVELIN lag
            lag = output['javelin']['tophat_lag'][1]
            lag_err_hi = output['javelin']['tophat_lag'][2]
            lag_err_lo = output['javelin']['tophat_lag'][0]
            good_ind = np.argwhere( ( ccf_lags >= lag-lag_err_lo ) & ( ccf_lags <= lag+lag_err_hi ) ).T[0]

            if len(good_ind) > 0:
                rmax_jav = np.max(ccf[good_ind])
            else:
                rmax_jav = ccf[ np.argmin( np.abs(lag - ccf_lags) ) ]

            summary_dict['rmax_javelin'] = rmax_jav
            output['rmax_javelin'] = rmax_jav

        else:
            summary_dict['rmax_javelin'] = np.nan
            output['rmax_javelin'] = np.nan


        if run_pyroa:
            #PyROA lag
            lag = output['pyroa']['time_delay'][1]
            lag_err_hi = output['pyroa']['time_delay'][2]
            lag_err_lo = output['pyroa']['time_delay'][0]
            good_ind = np.argwhere( ( ccf_lags >= lag-lag_err_lo ) & ( ccf_lags <= lag+lag_err_hi ) ).T[0]

            if len(good_ind) > 0:
                rmax_pyroa = np.max(ccf[good_ind])
            else:
                rmax_pyroa = ccf[ np.argmin( np.abs(lag - ccf_lags) ) ]

            summary_dict['rmax_pyroa'] = rmax_pyroa
            output['rmax_pyroa'] = rmax_pyroa
        else:
            summary_dict['rmax_pyroa'] = np.nan
            output['rmax_pyroa'] = np.nan


    else:
        summary_dict['rmax_pyccf'] = np.nan
        output['rmax_pyccf'] = np.nan

        summary_dict['rmax_javelin'] = np.nan
        output['rmax_javelin'] = np.nan

        summary_dict['rmax_pyroa'] = np.nan
        output['rmax_pyroa'] = np.nan


    #---------------------------
    #Write summary file
    write_weighting_summary(output_dir + 'weight_summary.fits', summary_dict, run_pyccf, run_javelin, run_pyroa)

    return output, summary_dict



import numpy as np

from pypetal.pyroa.plotting import (plot_fits, plot_histograms,
                                    pyroa_corner_plot, pyroa_trace_plot)
from pypetal.pyroa.utils import run_pyroa
from pypetal.utils import defaults


def pyroa_tot(cont_fname, line_fnames, line_names, output_dir,
              general_kwargs, kwargs):

   #--------------------------------------------------
    #Read general kwargs

    verbose = general_kwargs['verbose']
    plot = general_kwargs['plot']
    time_unit = general_kwargs['time_unit']
    lc_unit = general_kwargs['lc_unit']
    lag_bounds = general_kwargs['lag_bounds']

    #--------------------------------------------------
    #Read kwargs

    nchain, nburn, init_tau, subtract_mean, div_mean, \
            add_var, delay_dist, psi_types, together, \
                objname, prior_func = defaults.set_pyroa( kwargs, len(line_names) )

    if verbose:

        txt_str = """
Running PyROA
----------------
nburn: {}
nchain: {}
init_tau: {}
subtract_mean: {}
div_mean: {}
add_var: {}
delay_dist: {}
psi_types: {}
together: {}
objname: {}
----------------
        """.format(nburn, nchain, init_tau, subtract_mean, div_mean, add_var,
                     delay_dist, psi_types, together, objname)

        print(txt_str)

    tot_fnames = np.hstack( [ [cont_fname], line_fnames ] )

    lc_dir = output_dir + 'pyroa_lcs/'
    if not together:
        line_dir = [ output_dir + x + '/pyroa/' for x in line_names[1:]]
    else:
        line_dir = output_dir + 'pyroa/'



    res = run_pyroa( tot_fnames, lc_dir, line_dir, line_names,
                           nburn, nchain, lag_bounds, init_tau,
                           together=together, subtract_mean=subtract_mean,
                           div_mean=div_mean, add_var=add_var,
                           delay_dist=delay_dist, psi_types=psi_types,
                           objname=objname, prior_func=prior_func, verbose=verbose)

    lc_fnames = [ lc_dir + objname + '_' + x + '.dat' for x in line_names ]

    if together:
        pyroa_trace_plot( res.samples, line_names, add_var=add_var,
                                delay_dist=delay_dist, nburn=nburn,
                                fname = output_dir + 'pyroa/trace_plot.pdf',
                                show=plot)

        plot_histograms( res.samples, line_names, nburn=nburn,
                               add_var=add_var, delay_dist=delay_dist,
                               fname= output_dir + 'pyroa/histogram_plot.pdf',
                               show=plot)

        pyroa_corner_plot( res.samples, line_names, nburn=nburn,
                                 add_var=add_var, delay_dist=delay_dist,
                                 split=False,
                                 fname = output_dir + 'pyroa/corner_plot.pdf',
                                 show=plot)

        plot_fits( lc_fnames, line_names, res.samples, res.models,
                         nburn=nburn, add_var=add_var, delay_dist=delay_dist,
                         psi_types=psi_types, delimiter=None,
                         time_unit=time_unit, lc_unit=lc_unit,
                         output_fname = output_dir + 'pyroa/fits_plot.pdf',
                         show=plot)


    else:

        for i, res_i in enumerate(res):
            names_i = [ line_names[0], line_names[i+1] ]
            fnames_i = [ lc_fnames[0], lc_fnames[i+1] ]

            pyroa_trace_plot( res_i.samples, names_i, add_var=add_var[i],
                                    delay_dist=delay_dist[i], nburn=nburn,
                                    fname = output_dir + line_names[i+1] + '/pyroa/trace_plot.pdf',
                                    show=plot)

            plot_histograms( res_i.samples, names_i, nburn=nburn,
                                   add_var=add_var[i], delay_dist=delay_dist[i],
                                   fname= output_dir + line_names[i+1] + '/pyroa/histogram_plot.pdf',
                                   show=plot)

            pyroa_corner_plot( res_i.samples, names_i, nburn=nburn,
                                     add_var=add_var[i], delay_dist=delay_dist[i],
                                     split=False,
                                     fname = output_dir + line_names[i+1] + '/pyroa/corner_plot.pdf',
                                     show=plot)

            plot_fits( fnames_i, names_i, res_i.samples, res_i.models,
                             nburn=nburn, add_var=add_var[i], delay_dist=delay_dist[i],
                             psi_types=psi_types[i], delimiter=None,
                             time_unit=time_unit, lc_unit=lc_unit,
                             output_fname = output_dir + line_names[i+1] + '/pyroa/fits_plot.pdf',
                             show=plot)


    return res



import glob
import os

import numpy as np
from astropy.table import Table, vstack

from pypetal.pyroa.utils import MyFit


def get_line_names(main_dir):
    dirs = glob.glob(main_dir + '*/')

    if main_dir + 'light_curves/' in dirs:
        dirs.remove(main_dir + 'light_curves/')

    if main_dir + 'processed_lcs/' in dirs:
        dirs.remove(main_dir + 'processed_lcs/')

    if main_dir + 'pyroa_lcs/' in dirs:
        dirs.remove(main_dir + 'pyroa_lcs/')

    if main_dir + 'javelin/' in dirs:
        if len(dirs) > 2:
            dirs.remove(main_dir + 'javelin/' )
            line_names = [ os.path.basename(os.path.dirname(x)) for x in dirs ]
        else:
            fits_files = glob.glob(main_dir + 'javelin/*_lc_fits.dat')
            line_names = [ os.path.basename(x[:-12]) for x in fits_files ]

    elif (main_dir + 'pyroa/' in dirs) & ( len(dirs) > 2 ):
        dirs.remove(main_dir + 'pyroa/' )
        line_names = [ os.path.basename(os.path.dirname(x)) for x in dirs ]

    else:
        line_names = [ os.path.basename(os.path.dirname(x)) for x in dirs ]

    return line_names


def str2bool(string):
    if string == 'False':
        return False
    elif string == 'True':
        return True


def get_modules(main_dir):

    line_names = get_line_names(main_dir)
    line_dirs = np.array([ main_dir + x + r'/' for x in line_names ])

    ###########################################################
    #DRW Rejection
    has_drw = np.zeros( len(line_names), dtype=bool )
    for i, dir_i in enumerate(line_dirs):
        subdirs = glob.glob(dir_i + '*/')
        if dir_i + 'drw_rej/' in subdirs:
            has_drw[i] = True

    run_drw_rej = np.any( has_drw )

    ###########################################################
    #Detrend

    has_detrend = np.zeros( len(line_names), dtype=bool )
    for i, dir_i in enumerate(line_dirs):
        subdirs = glob.glob(dir_i + '*.pdf')
        if dir_i + 'detrend.pdf' in subdirs:
            has_detrend[i] = True

    run_detrend = np.any( has_detrend )


    #Get number of lines (not counting continuum)
    n_lnc = len(line_names) - 1

    ###########################################################
    #pyCCF
    has_pyccf = np.zeros( len(line_names), dtype=bool )
    for i, dir_i in enumerate(line_dirs):
        subdirs = glob.glob(dir_i + '*/')
        if dir_i + 'pyccf/' in subdirs:
            has_pyccf[i] = True


    n_pyccf = len( np.argwhere(has_pyccf).T[0] )
    if ( n_pyccf != n_lnc ) & ( n_pyccf > 0 ):
        print('pyCCF was not completed for all lines, so will assume run_pyccf=False')
        run_pyccf = False
    else:
        run_pyccf = np.any( has_pyccf )


    ###########################################################
    #pyZDCF
    has_pyzdcf = np.zeros( len(line_names), dtype=bool )
    for i, dir_i in enumerate(line_dirs):
        subdirs = glob.glob(dir_i + '*/')
        if dir_i + 'pyzdcf/' in subdirs:
            has_pyzdcf[i] = True


    n_pyzdcf = len( np.argwhere(has_pyzdcf).T[0] )
    if ( n_pyzdcf != n_lnc ) & ( n_pyzdcf > 0 ):
        print('pyZDCF was not completed for all lines, so will assume run_pyzdcf=False')
        run_pyzdcf = False
    else:
        run_pyzdcf = np.any( has_pyzdcf )


    ###########################################################
    #PyROA
    run_pyroa = False

    dirs_tot = glob.glob(main_dir + '*/')
    if main_dir + 'pyroa/' in dirs_tot:
        run_pyroa = True

    else:
        has_pyroa = np.zeros( len(line_names), dtype=bool )
        for i,dir_i in enumerate(line_dirs):
            subdirs = glob.glob(dir_i + '*/')

            if dir_i + 'pyroa/' in subdirs:
                has_pyroa[i] = True

        n_pyroa = len( np.argwhere(has_pyroa).T[0] )
        if ( n_pyroa != n_lnc ) & ( n_pyroa > 0 ):
            print('PyROA was not completed for all lines, so will assume run_pyroa=False')
        else:
            run_pyroa = np.any( has_pyroa )


    ###########################################################
    #JAVELIN
    run_javelin = False

    dirs_tot = glob.glob(main_dir + '*/')
    if main_dir + 'javelin/' in dirs_tot:
        run_javelin = True

    else:
        has_javelin = np.zeros( len(line_names), dtype=bool )
        for i,dir_i in enumerate(line_dirs):
            subdirs = glob.glob(dir_i + '*/')

            if dir_i + 'javelin/' in subdirs:
                has_javelin[i] = True

        n_javelin = len( np.argwhere(has_javelin).T[0] )
        if ( n_javelin != n_lnc ) & ( n_javelin > 0 ):
            print('JAVELIN was not completed for all lines, so will assume run_javelin=False')
        else:
            run_javelin = np.any( has_javelin )



    ###########################################################
    #Weighting
    has_weighting = np.zeros( len(line_names), dtype=bool )
    for i, dir_i in enumerate(line_dirs):
        subdirs = glob.glob(dir_i + '*/')

        if dir_i + 'weights/' in subdirs:
            has_weighting[i] = True

    n_weighting = len( np.argwhere(has_weighting).T[0] )
    if ( n_weighting != n_lnc ) & ( n_weighting > 0 ):
        print('Weighting was not completed for all lines, so will assume run_weighting=False')
        run_weighting = False
    else:
        run_weighting = np.any( has_weighting )



    return run_drw_rej, run_pyccf, run_pyzdcf, run_pyroa, run_javelin, run_weighting



def get_javelin_together(main_dir):


    dirs_tot = glob.glob( main_dir + '*/')

    dirs_tot.remove( main_dir + 'light_curves/')
    if main_dir + 'processed_lcs/' in dirs_tot:
        dirs_tot.remove( main_dir + 'processed_lcs/')

    if main_dir + 'javelin/' in dirs_tot:
        together = True
    else:
        together = False

    return together


def get_pyroa_together(main_dir):

    dirs_tot = glob.glob( main_dir + '*/')

    dirs_tot.remove( main_dir + 'light_curves/')
    if main_dir + 'processed_lcs/' in dirs_tot:
        dirs_tot.remove( main_dir + 'processed_lcs/')

    if main_dir + 'pyroa/' in dirs_tot:
        together = True
    else:
        together = False

    return together



def get_cont_name(main_dir):

    run_drw_rej, run_pyccf, run_pyzdcf, run_pyroa, run_javelin, _ = get_modules(main_dir)

    dirs_tot = glob.glob( main_dir + '*/')
    dirs_tot.remove( main_dir + 'light_curves/')
    if main_dir + 'processed_lcs/' in dirs_tot:
        dirs_tot.remove( main_dir + 'processed_lcs/')

    if run_pyroa:
        pyroa_together = get_pyroa_together(main_dir)
        dirs_tot.remove( main_dir + 'pyroa_lcs/')
        if pyroa_together:
            dirs_tot.remove( main_dir + 'pyroa/')

    if run_javelin:
        javelin_together = get_javelin_together(main_dir)
        if javelin_together:
            dirs_tot.remove( main_dir + 'javelin/')






    if run_pyccf:

        has_pyccf = np.zeros( len(dirs_tot), dtype=bool )
        for i, dir_i in enumerate(dirs_tot):
            subdirs = glob.glob(dir_i + '*/')
            if dir_i + 'pyccf/' in subdirs:
                has_pyccf[i] = True

        ind = np.argwhere( ~has_pyccf ).T[0][0]

        cont_name = os.path.basename( os.path.dirname( dirs_tot[ind] ) )


    elif run_pyzdcf:

        has_pyzdcf = np.zeros( len(dirs_tot), dtype=bool )
        for i, dir_i in enumerate(dirs_tot):
            subdirs = glob.glob(dir_i + '*/')
            if dir_i + 'pyzdcf/' in subdirs:
                has_pyzdcf[i] = True

        ind = np.argwhere( ~has_pyzdcf ).T[0][0]

        cont_name = os.path.basename(  os.path.dirname( dirs_tot[ind] )  )


    elif run_pyroa & (not pyroa_together):

        has_pyroa = np.zeros( len(dirs_tot), dtype=bool )
        for i, dir_i in enumerate(dirs_tot):
            subdirs = glob.glob(dir_i + '*/')

            if dir_i + 'pyroa/' in subdirs:
                has_pyroa[i] = True

        ind = np.argwhere( ~has_pyroa ).T[0][0]
        cont_name = os.path.basename( os.path.dirname( dirs_tot[ind] ) )



    elif run_javelin:
        together = get_javelin_together(main_dir)

        if together:
            lc_files = glob.glob( main_dir + 'light_curves/*.dat' )
            cont_dat = np.loadtxt( main_dir + 'javelin/cont_lcfile.dat', unpack=True, usecols=[0,1,2] )

            xcont = cont_dat.jlist[0]
            ycont = cont_dat.mlist[0] + cont_dat.blist[0]
            yerrcont = cont_dat.elist[0]
            for f in lc_files:
                xline, yline, yerrline = np.loadtxt(f, unpack=True, delimiter=',', usecols=[0,1,2])

                if np.all( xline == xcont ) & np.all( yline == ycont ) & np.all( yerrline == yerrcont ):
                    cont_name = os.path.basename(f)[:-4]
                    break


        else:

            has_javelin = np.zeros( len(dirs_tot), dtype=bool )
            for i, dir_i in enumerate(dirs_tot):
                subdirs = glob.glob(dir_i + '*/')

                if dir_i + 'javelin/' in subdirs:
                    has_javelin[i] = True

            ind = np.argwhere( ~has_javelin ).T[0][0]
            cont_name = os.path.basename( os.path.dirname( dirs_tot[ind] ) )



    elif run_drw_rej:
        line_names, reject_data = get_reject_data(main_dir, ordered=False)

        if np.all(~reject_data):
            cont_name = line_names[0]
        else:
            possible_names = np.array(line_names)[reject_data]

            cont_name = possible_names[0]

    else:
        line_names = get_line_names(main_dir)
        cont_name = line_names[0]

    return cont_name



def get_ordered_line_names(main_dir):
    line_names = get_line_names(main_dir)
    cont_name = get_cont_name(main_dir)

    line_names.remove(cont_name)

    line_names_tot = np.hstack([ [cont_name], line_names ])

    return line_names_tot


#######################################################################
#                            DRW REJECTION
#######################################################################


def get_reject_data(main_dir, ordered=True):

    if ordered:
        line_names = get_ordered_line_names(main_dir)
    else:
        line_names = get_line_names(main_dir)

    reject_data = np.zeros( len(line_names), dtype=bool )
    for i, name in enumerate(line_names):
        dir_i = main_dir + name + r'/'

        subdirs = glob.glob(dir_i + '*/')
        if dir_i + 'drw_rej/' in subdirs:
            reject_data[i] = True

    return line_names, reject_data


def load_drw_rej(dir_loc):

    line_names = get_ordered_line_names(dir_loc)
    line_dirs = np.array([ dir_loc + x + r'/' for x in line_names ])

    res_dict = {}
    res_dict['taus'] = []
    res_dict['sigmas'] = []
    res_dict['jitters'] = []

    res_dict['fit_x'] = []
    res_dict['fit_y'] = []
    res_dict['fit_err'] = []

    res_dict['masks'] = []
    res_dict['reject_data'] = get_reject_data(dir_loc)[1]
    res_dict['names'] = []

    for i, dir_i in enumerate(line_dirs):

        if res_dict['reject_data'][i]:
            chain_dat = Table.read( dir_i + 'drw_rej/' + line_names[i] + '_chain.dat',
                                   format='ascii.csv')

            res_dict['taus'].append( np.array(chain_dat['tau']) )
            res_dict['sigmas'].append( np.array(chain_dat['#sigma']) )

            if 'jitter' in chain_dat.colnames:
                res_dict['jitters'].append( np.array(chain_dat['jitter']) )
            else:
                res_dict['jitters'].append(None)


            x, y, yerr = np.loadtxt( dir_i + 'drw_rej/' + line_names[i] + '_drw_fit.dat', unpack=True, delimiter=',', usecols=[0,1,2] )
            res_dict['fit_x'].append(x)
            res_dict['fit_y'].append(y)
            res_dict['fit_err'].append(yerr)

            mask = np.loadtxt( dir_i + 'drw_rej/' + line_names[i] + '_mask.dat', dtype=str)
            mask = list( map(str2bool, mask) )
            res_dict['masks'].append(mask)

            res_dict['names'].append(line_names[i])

        else:
            res_dict['taus'].append(None)
            res_dict['sigmas'].append(None)
            res_dict['jitters'].append(None)

            res_dict['fit_x'].append(None)
            res_dict['fit_y'].append(None)
            res_dict['fit_err'].append(None)


            x, y, yerr = np.loadtxt( dir_loc + 'light_curves/' + line_names[i] + '.dat', unpack=True, delimiter=',', usecols=[0,1,2] )
            res_dict['masks'].append( np.zeros( len(x), dtype=bool ) )
            res_dict['names'].append(line_names[i])

    return res_dict


#######################################################################
#                              PYCCF
#######################################################################


def load_pyccf(dir_loc):

    line_names = get_ordered_line_names(dir_loc)
    dirs_tot = np.array([ dir_loc + x + r'/' for x in line_names ])

    line_dirs = dirs_tot[1:]

    res_dict_tot = []
    for i, dir_i in enumerate(line_dirs):
        pyccf_dir = dir_i + 'pyccf/'
        dict_i = {}

        cccd_dist, ccpd_dist = np.loadtxt( pyccf_dir + line_names[i+1] + '_ccf_dists.dat', delimiter=',', unpack=True, usecols=[0,1])
        dict_i['CCCD_lags'] = cccd_dist
        dict_i['CCPD_lags'] = ccpd_dist

        lags, ccf = np.loadtxt( pyccf_dir + line_names[i+1] + '_ccf.dat', delimiter=',', unpack=True, usecols=[0,1])
        dict_i['CCF_lags'] = lags
        dict_i['CCF'] = ccf

        dict_i['name'] = line_names[i+1]

        res_dict_tot.append(dict_i)

    return res_dict_tot


#######################################################################
#                             PYZDCF
#######################################################################

def get_run_plike(line_dir):
    pyzdcf_dir = line_dir + 'pyzdcf/'

    line_name = os.path.basename( os.path.dirname(line_dir) )

    if pyzdcf_dir + line_name + '_plike.out' in glob.glob( pyzdcf_dir + '*' ):
        return True
    else:
        return False


def load_pyzdcf(dir_loc):

    line_names = get_ordered_line_names(dir_loc)
    dirs_tot = np.array([ dir_loc + x + r'/' for x in line_names ])

    line_dirs = dirs_tot[1:]

    pyzdcf_dict_tot = []
    plike_dict_tot = []
    for i, dir_i in enumerate(line_dirs):
        pyzdcf_dir = dir_i + 'pyzdcf/'
        dict_i = {}

        dcf_file = glob.glob(pyzdcf_dir + '*.dcf')[0]
        zdcf_df = Table.read(dcf_file, format='ascii',
                             names=['tau', '-sig(tau)', '+sig(tau)', 'dcf', '-err(dcf)', '+err(dcf)', '#bin']).to_pandas()


        dict_i['output'] = zdcf_df
        dict_i['name'] = line_names[i+1]

        pyzdcf_dict_tot.append(dict_i)


        run_plike = get_run_plike(dir_i)
        if run_plike:
            dict_i = {}

            plike_file = glob.glob(pyzdcf_dir + '*.out')[0]
            plike_df = Table.read(plike_file, format='ascii',
                                  names=['num', 'lag', 'r', '-dr', '+dr', 'likelihood'])


            with open(plike_file, 'r') as file:
                output_str = list(file)[-3:]

                ml_lag = float( output_str[1].split()[7] )
                ml_lag_err_hi = np.abs( float( output_str[1].split()[8] )  )
                ml_lag_err_lo = np.abs( float( output_str[1].split()[9] )  )


            dict_i = {
                'output': plike_df,
                'ML_lag': ml_lag,
                'ML_lag_err_hi': ml_lag_err_hi,
                'ML_lag_err_lo': ml_lag_err_lo,
                'name': line_names[i+1]
            }
            plike_dict_tot.append(dict_i)

        else:
            plike_dict_tot.append({})

    return pyzdcf_dict_tot, plike_dict_tot




#######################################################################
#                             PYZDCF
#######################################################################

def load_pyroa(dir_loc):

    line_names = get_ordered_line_names(dir_loc)
    dirs_tot = np.array([ dir_loc + x + r'/' for x in line_names ])
    together = get_pyroa_together(dir_loc)

    line_dirs = dirs_tot[1:]


    if together:

        res = MyFit( dir_loc + 'pyroa/' )

        return res

    else:

        res_tot = []
        for i in range(len(line_dirs)):
            res = MyFit( line_dirs[i] + 'pyroa/' )
            res_tot.append(res)

        return res_tot

#######################################################################
#                             JAVELIN
#######################################################################

def load_javelin(dir_loc):

    line_names = get_ordered_line_names(dir_loc)
    dirs_tot = np.array([ dir_loc + x + r'/' for x in line_names ])
    together = get_javelin_together(dir_loc)

    line_dirs = dirs_tot[1:]


    if together:
        res_dict = {}

        javelin_dir = dir_loc + 'javelin/'

        chains = np.loadtxt( javelin_dir + 'chain_rmap.txt', unpack=True )
        res_dict['tau'] = np.exp(chains[1])
        res_dict['sigma'] = np.exp(chains[0])
        res_dict['tophat_params'] = chains[2:]

        x, y, yerr = np.loadtxt(javelin_dir + line_names[0] + '_lc_fits.dat', unpack=True, delimiter=',', usecols=[0,1,2])
        res_dict['cont_fit_x'] = x
        res_dict['cont_fit_y'] = y
        res_dict['cont_fit_yerr'] = yerr

        for i in range(1, len(line_names)):
            x, y, yerr = np.loadtxt(javelin_dir + line_names[i] + '_lc_fits.dat', unpack=True, delimiter=',', usecols=[0,1,2])
            res_dict[line_names[i] + '_fit_x'] = x
            res_dict[line_names[i] + '_fit_y'] = y
            res_dict[line_names[i] + '_fit_yerr'] = yerr

        return res_dict


    else:

        res_dict_tot = []
        for i, dir_i in enumerate(line_dirs):
            javelin_dir = dir_i + 'javelin/'
            dict_i = {}


            chains = np.loadtxt( javelin_dir + 'chain_rmap.txt', unpack=True )
            dict_i['tau'] = np.exp(chains[1])
            dict_i['sigma'] = np.exp(chains[0])
            dict_i['tophat_params'] = chains[2:]

            x, y, yerr = np.loadtxt(javelin_dir + line_names[0] + '_lc_fits.dat', unpack=True, delimiter=',', usecols=[0,1,2])
            dict_i['cont_fit_x'] = x
            dict_i['cont_fit_y'] = y
            dict_i['cont_fit_yerr'] = yerr

            x, y, yerr = np.loadtxt(javelin_dir + line_names[i+1] + '_lc_fits.dat', unpack=True, delimiter=',', usecols=[0,1,2])
            dict_i['fit_x'] = x
            dict_i['fit_y'] = y
            dict_i['fit_yerr'] = yerr

            dict_i['name'] = line_names[i+1]
            res_dict_tot.append(dict_i)


        return res_dict_tot


#######################################################################
#                             Weighting
#######################################################################

def str2num(string, dtype=float, arr=False):

    if string == 'None':
        return None

    if arr:
        string = string.strip('[ ]')
        string = string.split(',')


    if dtype == float:
        if arr:
            return list(map(float, string))
        else:
            return float(string)

    elif dtype == int:
        if arr:
            return list(map(int, string))
        else:
            return int(string)



def load_weighting(main_dir, run_pyccf, run_pyroa, run_javelin):

    line_names = get_ordered_line_names(main_dir)
    dirs_tot = np.array([ main_dir + x + r'/' for x in line_names ])

    line_dirs = dirs_tot[1:]

    output = {
        'pyccf': {},
        'javelin': {},
        'pyroa': {},
        'rmax_pyccf': [],
        'rmax_pyroa': [],
        'rmax_javelin': [],
        'names': []
    }

    if run_pyccf:
        output['pyccf']['centroid'] = []
        output['pyccf']['bounds'] = []
        output['pyccf']['acf'] = []
        output['pyccf']['lags'] = []
        output['pyccf']['weight_dist'] = []
        output['pyccf']['smoothed_dist'] = []
        output['pyccf']['ntau'] = []
        output['pyccf']['downsampled_CCCD'] = []
        output['pyccf']['frac_rejected'] = []



    if run_javelin:
        output['javelin']['tophat_lag'] = []
        output['javelin']['bounds'] = []
        output['javelin']['acf'] = []
        output['javelin']['lags'] = []
        output['javelin']['weight_dist'] = []
        output['javelin']['smoothed_dist'] = []
        output['javelin']['ntau'] = []
        output['javelin']['downsampled_lag_dist'] = []
        output['javelin']['frac_rejected'] = []



    if run_pyroa:
        output['pyroa']['time_delay'] = []
        output['pyroa']['bounds'] = []
        output['pyroa']['acf'] = []
        output['pyroa']['lags'] = []
        output['pyroa']['weight_dist'] = []
        output['pyroa']['smoothed_dist'] = []
        output['pyroa']['ntau'] = []
        output['pyroa']['downsampled_lag_dist'] = []
        output['pyroa']['frac_rejected'] = []




    output['names'] = []


    res_dicts_tot = []
    for i, dir_i in enumerate(line_dirs):
        weight_dir = dir_i + 'weights/'
        summary_dict = Table.read(weight_dir + 'weight_summary.fits')

        res_dicts_tot.append(summary_dict)

        if run_javelin:
            lag_err_lo = summary_dict['lag_err_javelin'][0][0]
            lag_err_hi = summary_dict['lag_err_javelin'][0][1]
            med_lag = summary_dict['lag_javelin'][0]
            output['javelin']['tophat_lag'].append([lag_err_lo, med_lag, lag_err_hi])

            min_bound = summary_dict['peak_bounds_javelin'][0][0]
            max_bound = summary_dict['peak_bounds_javelin'][0][1]
            peak = summary_dict['peak_javelin'][0]
            output['javelin']['bounds'].append([min_bound, peak, max_bound])

            lags, ntau, weight_dist, acf, smooth_dist, smooth_weight_dist = np.loadtxt( weight_dir + 'javelin_weights.dat', delimiter=',', unpack=True )
            output['javelin']['lags'].append(lags)
            output['javelin']['ntau'].append(ntau)
            output['javelin']['weight_dist'].append(weight_dist)
            output['javelin']['acf'].append(acf)
            output['javelin']['smoothed_dist'].append(smooth_weight_dist)

            weighted_lag_dist = np.loadtxt( weight_dir + 'javelin_weighted_lag_dist.dat', delimiter=',', unpack=True )
            output['javelin']['downsampled_lag_dist'].append(weighted_lag_dist)

            output['javelin']['frac_rejected'].append( summary_dict['frac_rejected_javelin'][0] )



        if run_pyroa:
            lag_err_lo = summary_dict['lag_err_pyroa'][0][0]
            lag_err_hi = summary_dict['lag_err_pyroa'][0][1]
            med_lag = summary_dict['lag_pyroa'][0]
            output['pyroa']['time_delay'].append([lag_err_lo, med_lag, lag_err_hi])

            min_bound = summary_dict['peak_bounds_pyroa'][0][0]
            max_bound = summary_dict['peak_bounds_pyroa'][0][1]
            peak = summary_dict['peak_pyroa'][0]
            output['pyroa']['bounds'].append([min_bound, peak, max_bound])

            lags, ntau, weight_dist, acf, smooth_dist, smooth_weight_dist = np.loadtxt( weight_dir + 'pyroa_weights.dat', delimiter=',', unpack=True )
            output['pyroa']['lags'].append(lags)
            output['pyroa']['ntau'].append(ntau)
            output['pyroa']['weight_dist'].append(weight_dist)
            output['pyroa']['acf'].append(acf)
            output['pyroa']['smoothed_dist'].append(smooth_weight_dist)

            weighted_lag_dist = np.loadtxt( weight_dir + 'pyroa_weighted_lag_dist.dat', delimiter=',', unpack=True )
            output['pyroa']['downsampled_lag_dist'].append(weighted_lag_dist)

            output['pyroa']['frac_rejected'].append( summary_dict['frac_rejected_pyroa'][0] )



        if run_pyccf:
            cent_err_lo = summary_dict['lag_err_pyccf'][0][0]
            cent_err_hi = summary_dict['lag_err_pyccf'][0][1]
            med_cent = summary_dict['lag_pyccf'][0]
            output['pyccf']['centroid'].append([cent_err_lo, med_cent, cent_err_hi])

            min_bound = summary_dict['peak_bounds_pyccf'][0][0]
            max_bound = summary_dict['peak_bounds_pyccf'][0][1]
            peak = summary_dict['peak_pyccf'][0]
            output['pyccf']['bounds'].append([min_bound, peak, max_bound])

            lags, ntau, weight_dist, acf, smooth_dist, smooth_weight_dist = np.loadtxt( weight_dir + 'pyccf_weights.dat', delimiter=',', unpack=True )
            output['pyccf']['lags'].append(lags)
            output['pyccf']['ntau'].append(ntau)
            output['pyccf']['weight_dist'].append(weight_dist)
            output['pyccf']['acf'].append(acf)
            output['pyccf']['smoothed_dist'].append(smooth_weight_dist)

            weighted_cccd = np.loadtxt( weight_dir + 'pyccf_weighted_cccd.dat', delimiter=',', unpack=True )
            output['pyccf']['downsampled_CCCD'].append(weighted_cccd)

            output['pyccf']['frac_rejected'].append( summary_dict['frac_rejected_pyccf'][0] )

            output['rmax_pyccf'].append( summary_dict['rmax_pyccf'][0])

            if run_pyroa:
                output['rmax_pyroa'].append( summary_dict['rmax_pyroa'][0] )

            if run_javelin:
                output['rmax_javelin'].append( summary_dict['rmax_javelin'][0] )


        output['names'].append( line_names[i+1] )

    #Stack summary dicts
    res_dicts_tot = vstack(res_dicts_tot)

    return output, res_dicts_tot


#######################################################################
#                             All Modules
#######################################################################

def load(main_dir, verbose=False):

    """Load the results from a previous run of pyPetal.

    Parameters
    ----------

    main_dir : str
        The main output directory of the pyPetal run.

    verbose : bool, optional
        Whether or not to print the summary of the run (i.e. what modules were run).




    Returns
    -------

    output : dict
        A dictionary containing the results of the run.

    """

    main_dir = os.path.abspath(main_dir) + r'/'

    run_drw_rej, run_pyccf, run_pyzdcf, run_pyroa, run_javelin, run_weighting = get_modules(main_dir)

    txt = """
Prior pyPetal run
---------------------
DRW Rejection: {}
pyCCF: {}
pyZDCF: {}
PyROA: {}
JAVELIN: {}
Weighting: {}
---------------------
""".format(run_drw_rej, run_pyccf, run_pyzdcf, run_pyroa, run_javelin, run_weighting)

    if verbose:
        print(txt)


    drw_rej_res = {}
    pyccf_res = {}
    pyzdcf_res = {}
    plike_res = {}
    pyroa_res = {}
    javelin_res = {}
    weighting_res = {}

    if run_drw_rej:
        drw_rej_res = load_drw_rej(main_dir)

    if run_pyccf:
        pyccf_res = load_pyccf(main_dir)

    if run_pyzdcf:
        pyzdcf_res, plike_res = load_pyzdcf(main_dir)

    if run_pyroa:
        pyroa_res = load_pyroa(main_dir)

    if run_javelin:
        javelin_res = load_javelin(main_dir)

    if run_weighting:
        weighting_res, summary_dicts = load_weighting(main_dir, run_pyccf, run_pyroa, run_javelin)

    res_tot = {}
    res_tot['drw_rej_res'] = drw_rej_res
    res_tot['pyccf_res'] = pyccf_res
    res_tot['pyzdcf_res'] = pyzdcf_res
    res_tot['plike_res'] = plike_res
    res_tot['pyroa_res'] = pyroa_res
    res_tot['javelin_res'] = javelin_res
    res_tot['weighting_res'] = weighting_res

    if run_weighting:
        return res_tot, summary_dicts
    else:
        return res_tot
